#!/usr/bin/env python
# -*- coding:utf-8 -*-

from lcyframe import route
from lcyframe import funts
from base import BaseHandler
from tornado.routing import *
@route("/demo", name="1ssf")
class DemoHandler(BaseHandler):
    """
    演示
    """

    @funts.params()
    def post(self):
        """添加
        测试post
        
        Request extra query params:
        - a # 角色id type: integer
        - b # 供应商id type: string
        - c # 手机号 type: string
        - d # 城市全拼列表 type: int
        - pic # 文件 type: file
        

        :return:
        :rtype:
        """
        # 上传文件 self.request.files = {'excel': [{'filename': 'xxxxxxx.xlsx', 'body':"xxxxxx"}]
        excel = self.params["excel"]  # {'filename': 'xxxxxxx.xlsx', 'body':"xxxxxx"}

        self.write_success()
        

    @funts.params()
    def get(self):
        """查看
        测试get

        Request extra query params:
        - a # 角色id type: integer
        - b # 供应商id type: string
        - d # 城市全拼列表 type: int


        :return:
        :rtype:
        """
        # 发送邮件
        # self.smtp.send_email(
        #     ["452809783@qq.com", "123220663@qq.com"],
        #     "邮件标题6",
        #     message="文本",
        #     html_message="""
        #         <p style="color:red">邮件文本内容</p>
        #         <p><a href="http://www.xxx.com">链接</a></p>
        #         <p>内容中的图片未被显示时，需添加发件人信任。或者点击邮件页面中的"显示图片 信任此发件人的图片"</p>
        #         <p><img src="https://www.runoob.com/wp-content/uploads/2016/04/smtp4.jpg"></p>
        #         <p>图片标签：以下需添加image1的标签图片</p>
        #         <p><img src="cid:image1"></p>
        #         <p>2222</p>
        #         """,
        #     images=["/Users/apple/Downloads/WX20220714-160954@2x.png"],
        #     attachs=["/Users/apple/Downloads/WX20220714-154222@2x.png"]
        # )
        # redis
        # self.redis.set("a", 1)
        # self.redis.get("a")

        # ssdb
        # self.ssdb.set("a", 1)
        # self.ssdb.get("a")

        # mq调用方法
        # self.mq.put({"event": "event1", "a": 1})

        # nsq调用方法
        # self.nsq.pub("test", '{"event": "on_create_user", "uid": 0}')
        # from producer.nsq import user
        # user.PublishUser().on_create_user({"uid": 0})
        # self.nsq.NsqEvent.on_create_user({"uid": 0})

        # mqtt调用方法,2种方法不可同时使用，否则会抢占链接
        # from producer.mqtt import user
        # user.MqttEvent().on_create_user({"a": "mqtt_test"})
        # self.mqtt.MqttEvent.on_create_user({"a": "mqtt_test"})

        # celery
        # self.celery.Events.save_image.delay(111)
        # self.celery.Events.UserEvent.register.delay(111)

        # beanstalk
        # self.beanstalk.ClassName.func({"DemoEvents": 1}, priority=65536, delay=0, ttr=60, tube="tube_name")
        # self.beanstalk.DefaultEvents.event({"DefaultEvents": 1})  # tube1
        # self.beanstalk.DemoEvents.event({"DemoEvents": 1})        # tube2

        # 上传文件 self.request.files = {'excel': [{'filename': 'xxxxxxx.xlsx', 'body':"xxxxxx"}]
        # excel = self.params["excel"]    # {'filename': 'xxxxxxx.xlsx', 'body':"xxxxxx"}

        # graylog
        # self.graylog.from_server(x="sdsdds")
        # 调用model，创建一条用户数据

        # self.model.DemoModel.get_demo(1)
        # self.websocket.Events.demo(**{"a": 1, "b": 2})

        # 开启事务
        # conn = self.mysql.begin()
        # # update时加锁，其他链接和线程阻塞等待，直到当前commit
        # count = self.mysql.execute("update demo2 set age =age+1 where age=30;", safe_conn=conn)
        # self.mysql.commit(conn)

        self.write_success({"a": 1})









#!/usr/bin/env python
# -*- coding:utf-8 -*-
import random, string
from test_script import *
from lcyframe.libs import cprint, utils


def get_admin_member(*args, **kwargs):
    """
    无说明:
    """
    headers = {"token": "111"}
    files = {}
    params = {
        "uid": 0,     # int,必填,uid
        }
    return send(methed="get", url="/admin/member", params=params, headers=headers, files=files)

def post_admin_member(*args, **kwargs):
    """
    无说明:
    """
    headers = {}
    files = {}
    params = {
        "user_name": "",     # str,必填,无描述
        "pass_word": "",     # str,必填,无说明，无说明
        "nick_name": "",     # str,必填,无描述
        "sex": 0,     # int,必填,1 无说明 0 无说明
        "mobile": "",     # str,必填,无说明
        "email": "",     # str,必填,无说明
        "gid": 0,     # int,必填,无说明 1 无说明 2 无说明
        }
    return send(methed="post", url="/admin/member", params=params, headers=headers, files=files)

def put_admin_member(*args, **kwargs):
    """
    无说明:
    """
    headers = {}
    files = {}
    params = {
        "uid": 0,     # int,必填,无描述
        "nick_name": "",     # str,选填,无描述
        "sex": 0,     # int,选填,1 无说明 0 无说明
        "mobile": "",     # str,选填,无说明
        "email": "",     # str,选填,无说明
        "gid": 0,     # int,选填,无说明 1 无说明 2 无说明
        }
    return send(methed="put", url="/admin/member", params=params, headers=headers, files=files)

def delete_admin_member(*args, **kwargs):
    """
    无说明:
    """
    headers = {}
    files = {}
    params = {
        "uid": 0,     # int,选填,uid
        }
    return send(methed="delete", url="/admin/member", params=params, headers=headers, files=files)


def post_admin_login(*args, **kwargs):
    """
    登录:
    """
    headers = {}
    files = {}
    params = {
        "user_name": "",     # str,必填,用户名
        "pass_word": "",     # str,必填,密码
        }
    return send(methed="post", url="/admin/login", params=params, headers=headers, files=files)


def post_admin_resetpwd(*args, **kwargs):
    """
    忘记密码:
    """
    headers = {}
    files = {}
    params = {
        "pass_word": "",     # str,必填,旧密码
        "新密码": "",     # str,必填,无说明
        }
    return send(methed="post", url="/admin/resetpwd", params=params, headers=headers, files=files)


def post_admin_find(*args, **kwargs):
    """
    无说明:无说明（无说明）
    """
    headers = {}
    files = {}
    params = {
        "page": "",     # str,必填,无说明
        }
    return send(methed="post", url="/admin/find", params=params, headers=headers, files=files)

def get_admin_find(*args, **kwargs):
    """
    无说明:无说明
    """
    headers = {}
    files = {}
    params = {
        }
    return send(methed="get", url="/admin/find", params=params, headers=headers, files=files)


def batch_groups():
    """
    测试簇
    一键测试
    """
    get_admin_member()       # 无说明
    post_admin_member()       # 无说明
    put_admin_member()       # 无说明
    delete_admin_member()       # 无说明
    post_admin_login()       # 登录
    post_admin_resetpwd()       # 忘记密码
    post_admin_find()       # 无说明
    get_admin_find()       # 无说明
    

if __name__ == "__main__":
    # 无说明
    get_admin_member()
    # 无说明
    # post_admin_member()
    # # 无说明
    # put_admin_member()
    # # 无说明
    # delete_admin_member()
    # # 登录
    # post_admin_login()
    # # 忘记密码
    # post_admin_resetpwd()
    # # 无说明
    # post_admin_find()
    # # 无说明
    # get_admin_find()
    


# coding=utf-8
import time, os, logging
import tornado.ioloop
from tornado.ioloop import IOLoop, PeriodicCallback
import nsq
from lcyframe.libs.nsq_route import NsqTask

class NsqWorker(object):
    def __init__(self, **kwargs):

        self.ROOT = kwargs["ROOT"]
        self.nsq_config = kwargs["nsq_config"]
        self.workers_path = self.nsq_config.get("workers_dir", ["works/nsq"])

        self.ioloop = tornado.ioloop.IOLoop.instance()
        self.wait = 1
        self.workers = NsqTask.get_workers(self.ROOT, self.workers_path)
        self.has_start_works = {}

    def connect(self, workers):
        for name, work in workers.items():
            self.has_start_works[name] = work

            logging.info("%s [topic %s, channel %s] listen success!" % (name, work.topic, work.channel))
            nsq.Reader(message_handler=work.message_handler,
                       nsqd_tcp_addresses=self.nsq_config["nsqd_tcp_addresses"],
                       topic=work.topic,
                       channel=work.channel,
                       lookupd_poll_interval=self.nsq_config.get("lookupd_poll_interval", 15))

    def _reload_work(self):
        """

        :return:
        """
        print("auto load working..")
        NsqTask.get_workers(self.ROOT, self.workers_path)
        new_works = {}
        for name, work in self.workers.items():
            if name not in self.has_start_works:
                new_works[name] = work

        if new_works:
            self.connect(new_works)

    def start(self):
        self.connect(self.workers)
        # PeriodicCallback(self._reload_work, 1*1000, io_loop=self.ioloop).start()
        self.ioloop.start()


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    kwargs = {"nsq_config": {
        "workers": ["workers"],
        "host": "127.0.0.1",
        "port": 6379,
        "name": "mq"
    }}
    wk = NsqWorker(**kwargs)
    wk.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.upload_schema import UploadSchema

class UploadModel(BaseModel, UploadSchema):

    @classmethod
    def get_upload(cls, upload_id, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        d = cls.find_one_by_oid(upload_id)
        return cls._parse_data(d)


    @classmethod
    def get_upload_by_spec(cls, spec, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        d = cls.find_one(spec)
        return cls._parse_data(d)


    @classmethod
    def get_upload_list_by_last_id(cls, last_id, count, **kwargs):
        """
        前端列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, last_id = cls.find_list_by_last_id(spec,
                                                      count,
                                                      sort=[("create_at", -1), ],
                                                      fields=kwargs.get("fields", None),
                                                      last_id_field=False)
        return [cls._parse_data(d) for d in data_list if d], last_id

    @classmethod
    def get_upload_list_by_page(cls, page, count, **kwargs):
        """
        后台列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, pages = cls.find_list_by_page(spec,
                                                 page,
                                                 count,
                                                 sort=[("create_at", -1), ],
                                                 fields=kwargs.get("fields", None))
        return [cls._parse_data(d) for d in data_list if d], pages

    @classmethod
    def create_upload(cls, *args, **kwargs):
        """
        创建
        :return:
        :rtype:
        """
        pass

    @classmethod
    def modify_upload(cls, *args, **kwargs):
        """
        修改
        :return:
        :rtype:
        """
        pass

    @classmethod
    def delete_upload(cls, upload_id, **kwargs):
        """
        删除
        :return:
        :rtype:
        """
        return cls.update({"_id": upload_id}, {"state": -1})

    @classmethod
    def _parse_data(cls, d, **kwargs):
        """
        组装单条数据
        :return:
        :rtype:
        """
        if not d:
            return {}

        d["upload_id"] = d.pop("_id", "")

        return d





# coding=utf-8
import os, re
import logging
import random
import time
from datetime import datetime, date
import zlib, hmac
from bson import BSON, ObjectId
from functools import wraps
from lcyframe.libs import utils, funts, cprint, JWT, aes, Rsa, errors
from utils import errors


def get_excel_url(host, img, path=None):
    """excel"""
    if path is None:
        path = "/data/excel"

    if "http" in host and host.startswith("http"):
        pass
    else:
        host = "http://" + host

    p = host + path
    p = os.path.join(p, img)
    return p


def get_imge_url(host, img, path=None):
    """******"""
    if path is None:
        path = "/data"

    if "http" in host and host.startswith("http"):
        pass
    else:
        host = "http://" + host

    p = os.path.join(host, path, img)
    return p


def save_filedata(data, base_path, **kwargs):
    """保存文件"""
    if not data:
        return "", ""

    filename = gen_filename(data, **kwargs)
    save_path = gen_filepath(base_path, filename)

    if not os.path.exists(base_path):
        os.mkdir(base_path)

    if not os.path.exists(save_path):
        with open(save_path, "wb") as f:
            body = data["body"] if isinstance(data["body"], bytes) else data["body"].encode("u8")
            f.write(body)

    return filename, save_path


def gen_filepath(base_path, filename=None):
    if not os.path.exists("%s" % base_path):
        os.makedirs("%s" % base_path)

    save_path = "%s/%s" % (base_path, filename)
    return save_path


def gen_filename(file_data, prefix="", suffix="", name_type="unique", require_format=[], require_size=0):
    """
    生成文件名
    根据数据md5对文件去重
    :param body:
    :param name_type: unique名称按数据去重，date 按时间格式命名 random 随机命名
    :return:

    """

    metadata = get_fileinfo(file_data, require_format=require_format, require_size=require_size)

    if name_type == "unique":
        filename = metadata["unique_name"]
    elif name_type == "date":
        filename = "%s-%s" % (datetime.now().strftime("%Y%m%d%H%M%S"), str(utils.random_int(4)))
    elif name_type == "random":
        filename = str(utils.gen_random_sint(16))
    else:
        filename = metadata["name"]

    if metadata["format"]:
        return "%s%s%s.%s" % (prefix, filename, suffix, metadata["format"])
    else:
        return "%s%s%s" % (prefix, filename, suffix)


def get_fileinfo(file_data=None, file_path=None, require_format=[], require_size=0):
    """
    读取文件基本信息
    :param file_data:
    :return:
    """
    if file_data:
        if not isinstance(file_data, dict):
            file_data = {
                "body": file_data,
                "filename": "default"
            }
        body = file_data["body"]
        filename = file_data["filename"]
        unique_name = utils.md5(body if isinstance(body, bytes) else body.encode("u8"))
        name, format = file_data["filename"].rsplit(".", 1) if "." in file_data["filename"] else (file_data["filename"], "")
        # TODO Fix 包含有以下几种符号的文件名，会导致系统解压命令失败：'('、')'。如需在线调用系统命令解压，请打开注释.
        #  推荐使用python的Patool库进行统一加解压，可避免此问题
        # if format in ["rar", "zip", "pkg", "gz", "tar", "jar", "7z"]:
        #     if name.find("(") or name.find(")"):
        #         name = name.replace("(", "（").replace(")", "）")
        #
        #     filename = name + "." + format if format else name

        size = len(body)
    elif file_path and os.path.exists(file_path):
        with open(file_path, "r") as p:
            body = p.read()
            unique_name = utils.md5(body)
            name, format = file_data.split("/")[-1].split(".")
            size = len(body)
    else:
        raise

    if require_format and format.lower() not in require_format:
        raise errors.UploadError("请上传指定格式：" + ",".join(require_format))
    if require_size and size > require_size:
        raise errors.UploadError("大小不能超过%sM" % int(require_size/1024/1024))

    return {
        "filename": filename,
        "name": name,
        "format": format.lower(),
        "size": size,
        "unique_name": unique_name,
        "unique_filename": ("%s.%s" % (unique_name, format)) if format else unique_name
    }


def app(permission_bit=0):
    """
    应用端访问： app web
    :param permission_bit:
        0： anybody、anonymous 游客、匿名用户
        1： login user登录用户
    :return:
    """
    def wrap(method):
        @wraps(method)
        def has_role(self, *args, **kwargs):
            self.params = funts.get_params(self)
            if self.application.app_config["wsgi"]["logging_debug"] == "debug":
                logging.warning(self.params)

            if self.request.headers.get("uid", None):
                _check_admin(self)
            else:
                raise errors.ErrorUserNotFound

            _check_token(self, permission_bit)

            return method(self, *args, **kwargs)

        return has_role

    return wrap

def admin(permission_bit=0):
    """
    管理后台验证: jtw
    支持多个客户端同时访问
    :param permission_bit:
        0： anybody、anonymous 游客、匿名用户
        1： login user登录用户
    :return:
    """

    def wrap(method):
        @wraps(method)
        def has_role(self, *args, **kwargs):
            self.uid = self.request.headers.get("uid", 0)
            if type(self.uid) != int and self.uid.isdigit():
                self.uid = int(self.uid)
            self.token = self.request.headers.get("token", "")
            if permission_bit:
                # 是否合法请求
                token_debug = self.application.token_config.get("debug", False)
                if not token_debug:
                    _check_token(self, permission_bit)
                
                # 用户后台
                if self.request.headers.get("uid", None):
                    _check_admin(self)
                
                # 开发者接口方式访问
                elif self.request.headers.get("appkey", None):
                    pass
                # 公司账户访问
                elif self.request.headers.get("commany", None):
                    pass

            self.params = funts.get_params(self)
            if self.app_config.get("logging_config", {}).get("level", "debug") == "debug":
                logging.debug(
                    "请求参数: " + (utils.to_json(utils.check2json(utils.pparams(self.request, params=self.params)))
                                if hasattr(self, "params") else str(utils.pparams(self.request) or {})))

            return method(self, *args, **kwargs)

        return has_role

    return wrap


def api(permission_bit=1):
    """
    开放平台api访问装饰器
    通常用于开发者远程调用
    Aes加密
    :param permission_bit:
        0： anybody、anonymous 游客、匿名用户
        1： login user登录用户
    :return:
    """

    def wrap(method):
        @wraps(method)
        def has_role(self, *args, **kwargs):
            self.params = funts.get_params(self)
            if self.application.app_config["wsgi"]["logging_debug"] == "debug":
                logging.warning(self.params)
                logging.warning(get_argument(self))

            self.uid = self.app_key = self.request.headers.get("appkey", None)
            self.token = self.request.headers.get("token", "")

            if not self.app_key or not self.token:
                raise errors.ErrorTokenInvalid

            if not ObjectId.is_valid(self.app_key):
                raise errors.ErrorTokenInvalid

            self.developer = self.model.DeveloperModel.get_developer(self.app_key)
            if not self.developer:
                raise errors.ErrorTokenInvalid

            self.developer_id = self.developer.get("_id", "")

            self.is_usdt = self.developer.get("is_usdt", False)

            # ******IP
            verifi_ip(self, self.developer["ip_list"])

            token_debug = self.application.token_config.get("debug", False)
            if not token_debug:
                if permission_bit != 0:
                    if not self.uid or not self.token:
                        raise errors.ErrorTokenInvalid

                    # ******
                    verifi_sign(self, self.token, self.developer["secret_key"])

            return method(self, *args, **kwargs)

        return has_role

    return wrap


def _check_admin(self):
    self.is_admin = True

    self.member = self.model.AdminModel.get_admin(self.uid)
    if not self.member:
        raise errors.ErrorUserNotFound

    self.gid = self.member["gid"]

def _check_token(self, permission_bit):
    if permission_bit != 0:
        if not self.uid or not self.token:
            raise errors.ErrorTokenInvalid

        jwt = JWT.JwtToken(self.application.token_config["secret"],
                           self.application.token_config["expire"])

        # if self.application.token_config["secret"] in ["******"]:
        #     return

        try:
            if not jwt.is_validate(self.token):
                raise errors.ErrorTokenInvalid

            if jwt.is_expire(self.token):
                raise errors.ErrorTokenExpireInvalid

            payload = jwt.decode(self.token)
            if payload and payload["uid"] != self.uid:
                raise errors.ErrorTokenInvalid
            # 下次请求的token
            self.set_header("Token", jwt.refresh_token(self.token))
        except Exception as e:
            raise errors.ErrorTokenInvalid

def check__forestall_brute(app, key):
    """
    防止暴力破解密码登录
    登录前判断
    """
    if not hasattr(app, "forestall_brute_mapping"):
        return True
    else:
        logs = app.forestall_brute_mapping.get(key, {"count": 0, "ts": 0})
        if logs and logs["count"] >= 10 and logs.get("expire") > utils.now():
            raise errors.ErrorInvalid("你的操作已被系统判定为攻击行为")
        else:
            return True

def add_forestall_brute(app, key):
    """
    防止暴力破解密码登录
    当密码错误后，将key加入规则
    如次数得到指定值，将限定一段时间内不能登录
    """
    if not hasattr(app, "forestall_brute_mapping"):
        setattr(app, "forestall_brute_mapping", {})
    logs = app.forestall_brute_mapping.get(key, {"count": 0, "ts": 0})
    logs["count"] += 1
    logs["ts"] = utils.now()
    if logs["count"] >= 10:
        logs["expire"] = utils.now() + 15*60
    app.forestall_brute_mapping[key] = logs

def remove_forestall_brute(app, key):
    """
    防止暴力破解密码登录,以下操作后，移除key
    登陆成功后
    修改密码后
    """
    if hasattr(app, "forestall_brute_mapping"):
        app.forestall_brute_mapping.pop(key, False)

def verifi_sign(self, token, secret_key):
    """
    验签
    :param self:
    :param token:
    :param secret_key:
    :return:
    """
    if not self.application.app_config["security"]["verifi_sign"]:
        return

    if secret_key in ["******"]:
        return

    argument = get_argument(self)

    # ******
    sign = encode_sign(secret_key, **{k: v[0] for k, v in self.request.body_arguments.items()})

    # ******
    new_sign = new_encode_sign(secret_key, **argument)

    new_sign2 = new_encode_sign2(secret_key, **argument)

    if token != sign and token != new_sign and token != new_sign2:
        raise errors.ErrorTokenInvalid


def verifi_ip(self, ip_list):
    """
    验证白名单
    :param self:
    :param ip_list:
    :return:
    """
    if not self.application.app_config["security"]["verifi_ip"]:
        return

    ip_list = [ip.encode("u8") for ip in ip_list]
    remote_ip = self.request.remote_ip.encode("u8")
    s = "*" in ip_list or remote_ip in ip_list
    if not s:
        raise errors.ErrorTranIpError


def encode_sign(secret_key, **kwargs):
    """
    验签 1.0
    :param secret_key:
    :param kwargs:
    :return:
    """
    mapping = {}
    for k, v in kwargs.items():
        mapping[k.lower()] = v

    keys = [k.lower() for k in mapping.keys()]
    keys.sort()
    s = "&".join(["%s=%s" % (k, mapping[k]) for k in keys])
    # TODO ******json******，******str='"sign"'******
    s = utils.to_json(s)
    s = aes.AES(secret_key).encode(s)
    s = hmac.HMAC(secret_key.encode("u8"), s).hexdigest()
    # s = utils.enbase64(s)
    return s


def new_encode_sign(secret_key, **kwargs):
    """
    验签 2.0
    :param secret_key:
    :param kwargs:
    :return:
    """
    mapping = {}
    for k, v in kwargs.items():
        mapping[k.lower()] = v

    keys = [k.lower() for k in mapping.keys()]
    keys.sort()
    s = "&".join(["%s=%s" % (k, mapping[k]) for k in keys])
    s = aes.AES(secret_key).encode(s)
    s = hmac.HMAC(secret_key.encode("u8"), s).hexdigest()
    # s = utils.enbase64(s)
    return s


def new_encode_sign2(secret_key, **kwargs):
    """
    验签 3.0
    :param secret_key:
    :param kwargs:
    :return:
    """
    mapping = {}
    for k, v in kwargs.items():
        mapping[k.lower()] = v

    keys = [k.lower() for k in mapping.keys()]
    keys.sort()
    s = "&".join(["%s=%s" % (k, str(mapping[k])) for k in keys])
    s = s + "&secret_key=" + secret_key

    token = utils.md5(s)
    return token


def get_token_sign_func(token_version):
    """
    token******
    :param token_version:
    :return:
    """
    mp = {
        "v1": new_encode_sign,
        "v2": new_encode_sign2
    }
    return mp[token_version]


def get_excel_path(root_path, file_name=None):
    if not file_name:
        file_name = "%s.xlsx" % (datetime.now().strftime("%Y%m%d%H%M%S") + str(random.randint(0, 99)))

    path = datetime.now().strftime("%Y%m%d")

    base_path = 'data/excel'
    if not os.path.exists("%s/%s/%s" % (root_path, base_path, path)):
        os.makedirs("%s/%s/%s" % (root_path, base_path, path))

    save_path = "%s/%s/%s/%s" % (root_path, base_path, path, file_name)

    file_name = '%s/%s' % (path, file_name)

    return save_path, file_name


def get_now_time_tuple(ts):
    d = utils.int_to_date_string(ts, fm="%d")
    m = utils.int_to_date_string(ts, fm="%m")
    y = utils.int_to_date_string(ts, fm="%Y")
    return y, m, d


def gen_sign_str(params):
    """
    ******
    :param params:
    :return:
    """
    keys = [k for k, v in params.items() if v != "" and k != "sign"]
    keys.sort()
    message = "&".join(["%s=%s" % (k, str(params[k])) for k in keys])

    return message


def pc_or_mobile(ua):
    """
    判断客户端PC还是手机端
    :param ua: ******User-Agent******
    :return:
    """
    factor = ua
    is_mobile = False
    _long_matches = r'googlebot-mobile|android|avantgo|blackberry|blazer|elaine|hiptop|ip(hone|od)|kindle|midp|mmp' \
                    r'|mobile|o2|opera mini|palm( os)?|pda|plucker|pocket|psp|smartphone|symbian|treo|up\.(browser|link)' \
                    r'|vodafone|wap|windows ce; (iemobile|ppc)|xiino|maemo|fennec'
    _long_matches = re.compile(_long_matches, re.IGNORECASE)
    _short_matches = r'1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)' \
                     r'|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)' \
                     r'|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw' \
                     r'|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8' \
                     r'|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit' \
                     r'|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)' \
                     r'|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji' \
                     r'|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|e\-|e\/|\-[a-w])|libw|lynx' \
                     r'|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(di|rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi' \
                     r'|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)' \
                     r'|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg' \
                     r'|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21' \
                     r'|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-' \
                     r'|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it' \
                     r'|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)' \
                     r'|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)' \
                     r'|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit' \
                     r'|wi(g |nc|nw)|wmlb|wonu|x700|xda(\-|2|g)|yas\-|your|zeto|zte\-'

    _short_matches = re.compile(_short_matches, re.IGNORECASE)

    if _long_matches.search(factor) != None:
        is_mobile = True
    user_agent = factor[0:4]
    if _short_matches.search(user_agent) != None:
        is_mobile = True

    return is_mobile


def get_ua_client(ua):
    """
    判断客户端是否为IOS
    :param ua:
    :return:
    """
    try:

        from user_agents import parse
        user_agent = parse(ua)

        if user_agent.is_pc:
            return 3
        else:
            if user_agent.os.family == 'iOS':
                return 1
            else:
                return 2
    except:
        return 0

def tts():
    return time.time() * 1000

def get_argument(self):
    """
    ******
    :param self:
    :return:
    """
    argument = {}
    try:
        keys = self.params.keys()
        for key in keys:
            if key == "secret_key":
                continue
            argument[key] = self.get_argument(key)
    except:
        argument = {k: v[0] for k, v in self.request.body_arguments.items() or self.request.arguments.items()}

    return argument





#!/usr/bin/env python
# -*- coding:utf-8 -*-
import random, string
from test_script import *
from lcyframe.libs import cprint, utils


def get_imagecode(*args, **kwargs):
    """
    获取验证码:
    """
    headers = {}
    files = {}
    params = {
        "uuid": "",     # str,必填,随机串
        }
    return send(methed="get", url="/imagecode", params=params, headers=headers, files=files)

def post_imagecode(*args, **kwargs):
    """
    图形验证码:
    """
    headers = {}
    files = {}
    params = {
        "stype": 0,     # int,选填,类型
        "uuid": "",     # str,必填,随机串
        "code": "",     # str,必填,图形验证码
        }
    return send(methed="post", url="/imagecode", params=params, headers=headers, files=files)


def get_smscode(*args, **kwargs):
    """
    获取短信验证码:
    """
    headers = {}
    files = {}
    params = {
        "stype": 0,     # int,选填,类型，预留
        "uuid": "",     # str,必填,随机串，预留
        "mobile": "",     # str,必填,手机号
        }
    return send(methed="get", url="/smscode", params=params, headers=headers, files=files)

def post_smscode(*args, **kwargs):
    """
    短信验证码:
    """
    headers = {}
    files = {}
    params = {
        "stype": 0,     # int,选填,类型，预留
        "uuid": "",     # str,必填,随机串，预留
        "mobile": "",     # str,必填,随机串
        "code": "",     # str,必填,验证码
        }
    return send(methed="post", url="/smscode", params=params, headers=headers, files=files)


def get_download(*args, **kwargs):
    """
    下载:下载资源,返回文件是binary数据
    """
    headers = {}
    files = {}
    params = {
        "stype": 0,     # int,必填,类型，1下载授权文件
        "name": "",     # str,必填,文件名
        }
    return send(methed="get", url="/download", params=params, headers=headers, files=files)


def get_update_status(*args, **kwargs):
    """
    当前升级状态:engine_status值，1正常；2准备；3就绪；4升级中
    """
    headers = {}
    files = {}
    params = {
        }
    return send(methed="get", url="/update_status", params=params, headers=headers, files=files)

def post_update_status(*args, **kwargs):
    """
    确认升级:
    """
    headers = {}
    files = {}
    params = {
        }
    return send(methed="post", url="/update_status", params=params, headers=headers, files=files)


def get_update_status_list(*args, **kwargs):
    """
    升级记录:
    """
    headers = {}
    files = {}
    params = {
        "page": 0,     # int,选填,翻页码
        "count": 0,     # int,选填,每页显示条数
        }
    return send(methed="get", url="/update_status/list", params=params, headers=headers, files=files)


def batch_groups():
    """
    测试簇
    一键测试
    """
    get_imagecode()       # 获取验证码
    post_imagecode()       # 图形验证码
    get_smscode()       # 获取短信验证码
    post_smscode()       # 短信验证码
    get_download()       # 下载
    get_update_status()       # 当前升级状态
    post_update_status()       # 确认升级
    get_update_status_list()       # 升级记录
    

if __name__ == "__main__":
    # 获取验证码
    get_imagecode()
    # 图形验证码
    post_imagecode()
    # 获取短信验证码
    get_smscode()
    # 短信验证码
    post_smscode()
    # 下载
    get_download()
    # 当前升级状态
    get_update_status()
    # 确认升级
    post_update_status()
    # 升级记录
    get_update_status_list()
    


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import random, string
from test_script import *
from lcyframe.libs import cprint, utils


def post_user(*args, **kwargs):
    """
    添加:测试post
    """
    headers = {}
    files = {}
    # 文件
    files["pic"] = open("../请指定文件路径")
    
    params = {
        "a": 0,     # integer,必填,角色id
        "b": "",     # string,选填,供应商id
        "c": "",     # string,选填,手机号
        "d": 0,     # int,选填,城市全拼列表
        }
    return send(methed="post", url="/user", params=params, headers=headers, files=files)

def get_user(*args, **kwargs):
    """
    查看:测试get
    """
    headers = {}
    files = {}
    params = {
        "a": 0,     # integer,必填,角色id
        "b": "",     # string,选填,供应商id
        "d": 0,     # int,选填,城市全拼列表
        }
    return send(methed="get", url="/user", params=params, headers=headers, files=files)


def post_user_list(*args, **kwargs):
    """
    添加:测试post
    """
    headers = {}
    files = {}
    # 文件
    files["pic"] = open("../请指定文件路径")
    
    params = {
        "a": 0,     # integer,必填,角色id
        "b": "",     # string,选填,供应商id
        "c": "",     # string,选填,手机号
        "d": 0,     # int,选填,城市全拼列表
        }
    return send(methed="post", url="/user_list", params=params, headers=headers, files=files)

def get_user_list(*args, **kwargs):
    """
    查看:测试get
    """
    headers = {}
    files = {}
    params = {
        "a": 0,     # integer,必填,角色id
        "b": "",     # string,选填,供应商id
        "d": 0,     # int,选填,城市全拼列表
        }
    return send(methed="get", url="/user_list", params=params, headers=headers, files=files)


def batch_groups():
    """
    测试簇
    一键测试
    """
    post_user()       # 添加
    get_user()       # 查看
    post_user_list()       # 添加
    get_user_list()       # 查看
    

if __name__ == "__main__":
    # 添加
    post_user()
    # 查看
    get_user()
    # 添加
    post_user_list()
    # 查看
    get_user_list()
    


# -*- coding: utf-8 -*-

import os
import hmac
import socket
import time
import json
import mimetypes
import functools
import hashlib
from urllib.parse import urlparse, urlencode
from base64 import urlsafe_b64encode
from tornado import gen, httpclient
from retry_class import RetryPolicy, CallWithRetryAsync
from tornado.httpclient import HTTPError


import requests

"""
Usage:
cow = Cow(ACCESS_KEY, SECRET_KEY)
b = cow.get_bucket(BUCKET)
b.put('a')
b.stat('a')
b.delete('a')
b.copy('a', 'c')
b.move('a', 'c')
"""




class QiniuRetryPolicy(RetryPolicy):
    """Define a retry policy that is adapted to the Amazon S3 service.
    Retries will only be attempted for HTTP 500-level errors, or if there
    was a basic network failure of some kind. By default, a request
    against S3 will be retried three times, with retries starting after
    at least 1/2 second, and exponentially backing off from there to a
    maximum of 10 seconds.
    """

    def __init__(self, max_tries=3, timeout=30, min_delay=.5, max_delay=10):
        RetryPolicy.__init__(self, max_tries=max_tries, timeout=timeout, min_delay=min_delay, max_delay=max_delay,
                             check_exception=RetryPolicy.AlwaysRetryOnException,
                             check_result=self._ShouldRetry)

    def _ShouldRetry(self, response):
        """Retry on:
          1. HTTP error codes 500 (Internal Server Error) and 503 (Service
             Unavailable).
          2. Tornado HTTP error code 599, which typically indicates some kind
             of general network failure of some kind.
          3. Socket-related errors.
        """
        if response.error:
            # Check for socket errors.

            if type(response.error) == socket.error or type(response.error) == socket.gaierror:
                return True

            # Check for HTTP errors.
            if isinstance(response.error, HTTPError):
                code = response.error.code
                if code in (500, 503, 599):
                    return True

        return False



class CowException(Exception):
    def __init__(self, url, status_code, content):
        self.url = url
        self.status_code = status_code
        self.content = content
        Exception.__init__(self, content)



def signing(secret_key, data):
    return urlsafe_b64encode(
        hmac.new(secret_key, data, hashlib.sha1).digest()
    )

def requests_error_handler(func):
    @functools.wraps(func)
    def deco(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except AssertionError as e:
            req = e.args[0]
            raise CowException(
                    req.url, req.status_code, req.content
                )
    return deco


class UploadToken(object):
    def __init__(self, access_key, secret_key, scope, ttl=3600, ops=None):
        self.access_key = access_key
        self.secret_key = secret_key
        self.scope = scope  # buket:filename
        self.ttl = ttl
        self._token = None
        self.generated_at = int(time.time())
        self.ops = ops      # 额外处理策略（预转换/持久换处理/视频裁图/音视频分片）

    @property
    def token(self):
        if int(time.time()) - self.generated_at > self.ttl - 60:
            # 还有一分钟也认为过期了， make new token
            self._token = None

        if not self._token:
            self._token = self._make_token()

        return self._token

    def _make_token(self):
        self.generated_at = int(time.time())
        info = {
            'scope': self.scope,
            'deadline': self.generated_at + self.ttl
        }

        if self.ops:
            info.update(self.ops)

        info = urlsafe_b64encode(json.dumps(info))
        token = signing(self.secret_key, info)
        return '%s:%s:%s' % (self.access_key, token, info)


class AccessToken(object):
    def __init__(self, access_key, secret_key, url, params=None):
        self.access_key = access_key
        self.secret_key = secret_key
        self.url = url
        self.params = params

        self.token = self.build_token()

    def build_token(self):
        uri = urlparse(self.url)
        token = uri.path
        if uri.query:
            token = '%s?%s' % (token, uri.query)
        token = '%s\n' % token
        if self.params:
            if isinstance(self.params, str):
                token += self.params
            else:
                token += urlencode(self.params)
        return '%s:%s' % (self.access_key, signing(self.secret_key, token))



class Cow(object):
    def __init__(self, **kwargs):
        self.up_host = kwargs["up_host"]
        self.rs_host = kwargs["rs_host"]
        self.rsf_host = kwargs["rsf_host"]
        self.access_key = kwargs["access_key"]
        self.secret_key = kwargs["secret_key"]

        self.retry_policy = kwargs.get("retry_policy", QiniuRetryPolicy())
        self.upload_tokens = {}

        self.stat = functools.partial(self._stat_rm_handler, 'stat')
        self.delete = functools.partial(self._stat_rm_handler, 'delete')
        self.copy = functools.partial(self._cp_mv_handler, 'copy')
        self.move = functools.partial(self._cp_mv_handler, 'move')

        # self.ops = {}

    def get_bucket(self, bucket):
        """对一个bucket的文件进行操作，
        推荐使用此方法得到一个bucket对象,
        然后对此bucket的操作就只用传递文件名即可
        """
        return Bucket(self, bucket)

    def generate_access_token(self, url, params=None):
        return AccessToken(self.access_key, self.secret_key, url, params=params).token


    def generate_upload_token(self, scope, ttl=3600, ops=None):
        """上传文件的uploadToken"""
        # if scope not in self.upload_tokens:     #　TODO 注意，同一个文件第一次生成后，token在进程内永久保留一个小时，如果在这个时间内上传同一个文件，获得的token是一样的，导致某些操作不能生效，请在获取后清掉这个self.upload_tokens[scope]
        #     self.upload_tokens[scope] = UploadToken(self.access_key, self.secret_key, scope, ttl=ttl, ops=self.ops)
        # return self.upload_tokens[scope].token

        # self.upload_tokens[scope] = UploadToken(self.access_key, self.secret_key, scope, ttl=ttl, ops=ops)
        # return self.upload_tokens[scope].token
        return UploadToken(self.access_key, self.secret_key, scope, ttl=ttl, ops=ops).token

    def build_requests_headers(self, token):
        return {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': 'QBox %s' % token
        }

    @gen.coroutine
    @requests_error_handler
    def api_call(self, url, params=None):
        token = self.generate_access_token(url, params=params)
        res = yield gen.Task(CallWithRetryAsync, self.retry_policy, self._post, url, headers=self.build_requests_headers(token), params=params)
        if res.code == 200:
            raise gen.Return(res.body)
        else:
            raise gen.Return(None)

    @gen.coroutine
    def list_buckets(self):
        """列出所有的buckets"""
        url = '%s/buckets' % self.rs_host
        result = yield self.api_call(url)
        raise gen.Return(result)

    def drop_bucket(self, bucket):
        """删除整个bucket"""
        url = '%s/drop/%s' % (self.rs_host, bucket)
        result = yield self.api_call(url)
        raise gen.Return(result)

    def list_files(self, bucket, marker=None, limit=None, prefix=None):
        """列出bucket中的文件"""
        query = ['bucket=%s' % bucket]
        if marker:
            query.append('marker=%s' % marker)
        if limit:
            query.append('limit=%s' % limit)
        if prefix:
            query.append('prefix=%s' % prefix)
        url = '%s/list?%s' % (self.rsf_host, '&'.join(query))
        result = yield self.api_call(url)
        raise gen.Return(result)

    @gen.coroutine
    @requests_error_handler
    def put(self, scope, data, filename=None, keep_name=True, override=True, content_type=None):
        """
        上传文件
        :param scope: bucket name
        :param filename: file name
        :param data: file data
        :param keep_name: if True, use file name, otherwise use the file data's md5 value
        :param override: if True, override the same key file.
        :return:
        """

        if not data:
            with open(filename, 'rb', encoding='utf-8') as f:
                data = f.read()

        if keep_name and filename:
            upload_name = filename
        else:
            upload_name = hashlib.md5(data).hexdigest()
            _, ext = os.path.splitext(filename)
            upload_name += ext


        if override:
            token = self.generate_upload_token('%s:%s' % (scope, upload_name))
        else:
            token = self.generate_upload_token(scope)

        files = {'file': data}
        action = '/rs-put/%s' % urlsafe_b64encode(
            '%s:%s' % (scope, upload_name)
        )
        if content_type:
            action += '/mimeType/%s' % urlsafe_b64encode(content_type)

        data = {
            'auth': token,
            'action': action,
        }

        a = requests.Request(url='%s/upload' % self.up_host, files=files, data=data)
        prepare = a.prepare()
        content_type = prepare.headers.get('Content-Type')

        res = yield gen.Task(CallWithRetryAsync, self.retry_policy, self._upload, prepare.body, content_type)
        raise gen.Return(res)

    @gen.coroutine
    def _upload(self, body, content_type):

        httpclient.AsyncHTTPClient.configure("tornado.curl_httpclient.CurlAsyncHTTPClient")
        http = httpclient.AsyncHTTPClient()
        response = yield http.fetch('%s/upload' % self.up_host,
                                    method='POST', body=body,
                                    headers={"Content-Type": content_type},
                                    connect_timeout=600,
                                    request_timeout=600)
        raise gen.Return(response)

    @gen.coroutine
    def _post(self, url, headers, params):

        httpclient.AsyncHTTPClient.configure("tornado.curl_httpclient.CurlAsyncHTTPClient")
        http = httpclient.AsyncHTTPClient()
        response = yield http.fetch(url,
                                    headers=headers,
                                    method='POST', body=params or "")
        raise gen.Return(response)

    def _cp_mv_handler(self, action, args):
        src_bucket, src_filename, des_bucket, des_filename = args
        url = '%s/%s/%s/%s' % (
            self.rs_host,
            action,
            urlsafe_b64encode('%s:%s' % (src_bucket, src_filename)),
            urlsafe_b64encode('%s:%s' % (des_bucket, des_filename)),
        )
        return self.api_call(url)


    def _stat_rm_handler(self, action, bucket, filename):
        url = '%s/%s/%s' % (
            self.rs_host, action, urlsafe_b64encode('%s:%s' % (bucket, filename))
        )
        return self.api_call(url)

class Bucket(object):
    def __init__(self, cow, bucket):
        self.cow = cow
        self.bucket = bucket

    def put(self, filename, data=None, keep_name=False, override=True, content_type=None):
        return self.cow.put(self.bucket, filename, data=data, keep_name=keep_name, override=override, content_type=content_type)

    def stat(self, filename):
        return self.cow.stat(self.bucket, filename)

    def delete(self, filename):
        return self.cow.delete(self.bucket, filename)

    def copy(self, *args):
        return self.cow.copy(self._build_cp_mv_args(*args))

    def move(self, *args):
        return self.cow.move(self._build_cp_mv_args(*args))

    def list_files(self, marker=None, limit=None, prefix=None):
        return self.cow.list_files(self.bucket, marker=marker, limit=limit, prefix=prefix)

    def _build_cp_mv_args(self, *args):
        return [self.bucket, args[0], self.bucket, args[1]]



class AdvanceUploadToken(UploadToken):
    def __init__(self, access_key, secret_key, info):
        self.access_key = access_key
        self.secret_key = secret_key
        self.info = info
        self._token = None

    @property
    def token(self):
        self.generated_at = int(time.time())
        info = urlsafe_b64encode(json.dumps(self.info))
        token = signing(self.secret_key, info)
        return '%s:%s:%s' % (self.access_key, token, info)


# coding=utf-8
import os
import logging
import json
import time
from lcyframe.libs import utils

class BsRoute(object):
    """消息队列路由"""
    _workers = {}

    def __init__(self, event=None):
        self.event = event

    def __call__(self, _handler):
        event = self.event and self.event or _handler.__name__
        _handler.from_route = True
        self._workers[event] = _handler
        return _handler

    @classmethod
    def tasks(cls, _handler):
        class_name = _handler.__name__
        if class_name in cls._workers:
            raise Exception("The event is exists multiple: `%s`" % class_name)

        cls._workers[class_name] = _handler
        return cls._workers

    @classmethod
    def get_workers(cls, ROOT, workers_path):

        if not isinstance(workers_path, list):
            workers_path = [workers_path]

        if workers_path is None:
            raise Exception("args workers_path is not allow empty")

        if not cls._workers:
            for work in workers_path:
                for root, dirs, files in os.walk(utils.fix_path(os.path.join(ROOT, work))):
                    for file in files:
                        if file.startswith("__"):
                            continue
                        if file.endswith(".pyc"):
                            continue
                        if not file.endswith(".py"):
                            continue
                        model_name = root.replace(ROOT, "").lstrip("/").replace("/", ".") + "." + file.rstrip(".py")
                        __import__(model_name, globals(), locals(), [model_name], 0)
                        logging.debug("register beanstalk workers [%s.py] success!" % model_name)

        return cls._workers

class Events(object):

    def __getattr__(self, event):
        event = "%s.%s" % (self.classname, event)
        if event not in self.__dict__:
            self.__dict__[event] = Agent(self.beanstalk, event, self.tube)

        return self.__dict__[event]


class Agent:

    def __init__(self, beanstalk, event, tube):
        self.beanstalk = beanstalk
        self.event = event
        self.tube = tube

    def __repr__(self):
        return "event '%s'" % self.event

    def __call__(self, body, priority=65536, delay=0, ttr=60):
        payload = {}
        payload["__event__"] = self.event  # message_handler name by Consumer
        payload["ts"] = int(time.time())
        payload.update(body)

        self.beanstalk.put(json.dumps(payload), priority=priority, delay=delay, ttr=ttr, tube=self.tube)




import os
from io import BytesIO
# from BaseColor.base_colors import hyellow, hred
from minio import Minio
import logging
from urllib.parse import urlparse


# def printer(name, text, *args):
#     log_ext = ' '.join([str(x) for x in args]) if args else ''
#     log_str = f"[{hyellow(name)}] {text} {log_ext}"
#     print(log_str)


class MinioOpt:
    """Minio操作类"""

    def __init__(self, host, port, access_key, secret_key, bucket_name=None, **kwargs):
        # 连接minio
        self.minioClient = Minio(f"{host}:{port}", access_key=access_key, secret_key=secret_key, secure=False)

        # # 初始化桶，不存在就创建
        # bucket_name = bucket_name.strip()
        # self._init_bucket()

    def _init_bucket(self, bucket_name):
        if bucket_name:
            try:
                if not self.minioClient.bucket_exists(bucket_name):
                    self.minioClient.make_bucket(bucket_name, location="cn-northFalse")
            except Exception as IBE:
                print(
                    "MINIO-init",
                    f"Error in checking minio bucket: {hyellow(bucket_name)}  -->{hred(IBE)}"
                )
                return False
            return True

    def save_file(self, bucket_name, object_name, path):
        """
        将文件上传到文件系统
        bucket: 存储桶
        object_name: 上传文件的名称：1.jpg
        path: 文件路径：/tmp/1.jpg

        return: etag: 对象的etag值。
        """
        try:
            etag = self.minioClient.fput_object(bucket_name, object_name, path)
        except Exception as err:
            return False, str(err), None

        return True, "", etag

    def save_data(self, bucket_name, object_name, data, length, content_type="application/octet-stream", metadata=None):
        """
        将文件对象上传到文件系统
        bucket: 存储桶
        object_name: 对象名
        data: io.RawIOBase, 任何实现了io.RawIOBase的python对象。
        length: 对象的总长度。
        content_type: 对象的Content type。
        path: 文件路径：/tmp/1.jpg
        metadata: dict, 其它元数据

        return: etag: 对象的etag值。

        外层使用姿势：
            bucket = 'my_bucket'
            object_name = 'my_object'
            minio_ins = MinioStorage()
            with open('my-testfile', 'rb') as file_data:
                file_stat = os.stat('my-testfile')
                minio_ins.save_data(bucket, object_name, file_data, file_stat.st_size)
        """


        try:
            etag = self.minioClient.put_object(bucket_name, object_name, data, length)
            return True, "", etag
        except Exception as err:
            return False, str(err), None

    def save_bytes(self, bucket_name, object_name, file_bytes, content_type=None, metadata=None):
        if isinstance(file_bytes, str):
            file_bytes = file_bytes.encode()
        file_length = len(file_bytes)
        io = BytesIO(file_bytes)
        io.seek(0)
        return self.save_data(bucket_name=bucket_name, object_name=object_name, data=io, length=file_length, content_type=content_type, metadata=metadata)

    def get_file_object(self, bucket_name, object_name):
        """
        获取文件对象
        bucket: 存储桶
        object_name: 上传文件的名称：1.jpg
        """
        try:
            file_object = self.minioClient.get_object(bucket_name, object_name, request_headers=None)
            return True, "", file_object
        except Exception as err:
            return False, str(err), None

    def load_file(self, bucket_name, object_name, file_path):
        """
        下载并将文件保存到本地
        bucket: 存储桶
        object_name: 上传文件的名称：1.jpg
        file_path: 要存储的本地文件路径：/tmp/1.jpg
        注意：本API支持的最大文件大小是5GB
        返回文件对象
        """
        try:
            file_object = self.minioClient.fget_object(bucket_name, object_name, file_path)
        except Exception as err:
            return False, str(err), None

        return True, "", file_object

    def remove_file(self, bucket_name, object_name):
        """
        删除存储文件
        bucket: 存储桶
        object_name: 上传文件的名称：1.jpg
        """
        try:
            self.minioClient.remove_object(bucket_name, object_name)
        except Exception as err:
            return False, str(err)

        return True, ""

    def remove_incomplete_file(self, bucket_name, object_name):
        """
        删除一个未完整上传的文件
        bucket: 存储桶
        object_name: 上传文件的名称：1.jpg
        """
        try:
            self.minioClient.remove_incomplete_upload(bucket_name, object_name)
        except Exception as err:
            return False, str(err)
        return True, ""

    def presigned_get_object(self, bucket_name, object_name):
        """
        生成查看URL
        """
        try:
            url = self.minioClient.presigned_get_object(bucket_name, object_name)
        except Exception as err:
            return False, str(err)

        parse = urlparse(url)
        return True, parse.path + "?" + parse.query

if __name__ == '__main__':
    if hasattr(os.environ, "app_config"):
        config = os.environ.app_config
        mino_config = config["MINIO"]
    else:
        mino_config = {"HOST": "10.96.128.45", "PORT": 9000, "ACCESS_KEY": "admin", "SECRET_KEY": "root123456",
                       "BUCKET_NAME": "html-bucket"}
    MINIO_HOST = mino_config["HOST"]
    MINIO_PORT = mino_config["PORT"]
    MINIO_ACCESS_KEY = mino_config["ACCESS_KEY"]
    MINIO_SECRET_KEY = mino_config["SECRET_KEY"]
    # MINIO_BUCKET_NAME = mino_config["BUCKET_NAME"]
    BUCKET_AVATAR = mino_config["BUCKET_AVATAR"]
    BUCKET_AUTHORIZEDOC = mino_config["BUCKET_AUTHORIZEDOC"]
    BUCKET_FOLLOWUP = mino_config["BUCKET_FOLLOWUP"]

    test_minio = MinioOpt(MINIO_HOST, MINIO_PORT, MINIO_ACCESS_KEY, MINIO_SECRET_KEY)

    # test_minio = MinioOpt(
    #     host="10.96.128.45",
    #     port=9000,
    #     access_key="admin",
    #     secret_key="root123456",
    #     bucket_name="html-bucket"
    # )

    # test_sta, test_err, test_tag = test_minio.save_file(
    #     object_name=""".env""",
    #     path="./../../conf/.env"
    # )
    # test_post_sta, test_post_err, test_post_tag = test_minio.save_bytes(
    #     object_name="""a.txt""",
    #     file_bytes="123ahwdgalibwliaubhwfliuabwfliybaliyf"
    # )
    # print("test_post_sta:", test_post_sta)
    # print("test_post_err:", test_post_err)
    # print("test_post_tag:", test_post_tag[0])

    # test_sta, test_err, test_tag = test_minio.get_file_object(
    #     "a.txt",
    # )
    # with open('my-testfile', 'wb') as file_data:
    #     for d in test_tag.stream(32 * 1024):
    #         file_data.write(d)

    # 上传授权文件
    with open('2d37df83750ac67caafaa2b83f51b93b.pdf', 'rb') as file_data:
        file_stat = os.stat('2d37df83750ac67caafaa2b83f51b93b.pdf')
        test_post_sta, test_post_err, test_post_tag = test_minio.save_data(
            bucket_name="authorizedoc",
            object_name="2d37df83750ac67caafaa2b83f51b93b.pdf",
            data=file_data,
            length=file_stat.st_size
        )
        print("test_post_sta:", test_post_sta)
        print("test_post_err:", test_post_err)
        print("test_post_tag:", test_post_tag)

    result = test_minio.presigned_get_authorizedoc("2d37df83750ac67caafaa2b83f51b93b.pdf")
    print(result)

    # 上传跟进文件
    with open('2d37df83750ac67caafaa2b83f51b93b.pdf', 'rb') as file_data:
        file_stat = os.stat('2d37df83750ac67caafaa2b83f51b93b.pdf')
        test_post_sta, test_post_err, test_post_tag = test_minio.save_data(
            bucket_name="followup",
            object_name="2d37df83750ac67caafaa2b83f51b93b.pdf",
            data=file_data,
            length=file_stat.st_size
        )
        print("test_post_sta:", test_post_sta)
        print("test_post_err:", test_post_err)
        print("test_post_tag:", test_post_tag)

    result = test_minio.presigned_get_followup("2d37df83750ac67caafaa2b83f51b93b.pdf")
    print(result)



import sys, os, shutil

def main():
    args = sys.argv
    cwd_path = os.getcwd()
    root_path = os.path.abspath(os.path.dirname(__file__))
    print(root_path)
    if len(args) == 1:
        file_name = args[0]
        file_path = cwd_path
    elif len(args) == 2:
        if os.path.isdir(args[1]):
            file_name = "demo"
            file_path = args[1]
        else:
            file_name = args[1]
            file_path = cwd_path
    elif len(args) == 3:
        file_name = args[1]
        file_path = args[2]
    else:
        file_name = "demo"
        file_path = cwd_path

    if not os.path.exists(file_path):
        print(" ")
        print("Error: No such file or directory '%s'" % file_path)
        print(" ")
        return

    if file_name == "demo" and os.path.exists(os.path.join(file_path, file_name)):
        file_name += "_lcyframe"

    if os.path.exists(os.path.join(file_path, file_name)):
        print(" ")
        print("Error: Directory already exists" + " '%s'" % os.path.join(file_path, file_name))
        print(" ")
        return

    shutil.copytree(os.path.join(root_path, 'demo'), os.path.join(file_path, file_name))

    if os.path.basename(cwd_path) == "lcyframe":
        pass
        # print("##################################################################################################")
        # print("#    Please run like this 'python -mlcyframe demo_name' in your directory to create a Demo !     #")
        # print("##################################################################################################")

    if os.path.exists(os.path.join(file_path, file_name)):
        print("##################################################################################################")
        print("#                                                                                                #")
        print("#     Created Successfully: '%s'" % os.path.join(file_path, file_name))
        print("#                                                                                                #")
        print("##################################################################################################")


if __name__ == '__main__':
    sys.exit(main())


#!/usr/bin/env python
# -*- coding:utf-8 -*-

from bson.objectid import ObjectId
from base import BaseSchema, utils
import time, datetime


class BasicsSchema(BaseSchema):
    """
    基础功能
    """

    collection = "basics"

    def __init__(self):
        self._id = ObjectId()




#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.demo_schema import DemoSchema

class DemoModel(BaseModel, DemoSchema):

    @classmethod
    def get_demo(cls, demo_id, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        data = cls.mysql.insert(cls.collection, {"name": 3}, new=True)
        # 获取当前表所有字段
        fields = DemoSchema.fields()

        # mongo

        # 新增记录（1条或者多条）
        # 1、框架的方法
        id = cls.insert({"uid": 102})
        ids = cls.insert([{"uid": 102}, {"uid": 103}], ordered=False)   # ordered并发写入，不保证顺序
        # 2、原生方法
        # cls.mongo.collection.insert_one({"uid": 102})
        # cls.mongo.collection.insert_many([{"uid": 102}, {"uid": 103}])
        
        # 删除
        # 删除单条
        # deleted_count = cls.remove({"uid": 103}, multi=False)
        # # 删除多条
        # deleted_count = cls.remove({"uid": 102})
        
        # 查看
        data = cls.find_one({"yyy": 102})
        datas = cls.find({"uid": 102}, fields={"uid": 1}, skip=1, limit=10, sort={"uid": 1})
        
        # 替换/覆盖/新增  result.matched_count,result.modified_count,result.upserted_id
        # result = cls.replace_one({"uid": 102}, {"uidx": 102})                             # 修改字段
        # result = cls.replace_one({"xxx": 100}, {"xxx": 100}, upsert=True)                 # 没有则新增
        # result = cls.replace_one({"xxx": 200}, {"yyy": 200, "zzz": 100}, upsert=True)     # 没有则新增一条，并新增一个字段
        
        # 更新数据
        # update_state = cls.update({"yyy": 200},cls.find {"$inc": {"xxx": 1}})
        # update_state = cls.update({"yyy": 200}, {"$inc": {"xxx": 1}}, multi=True)
        # <class 'dict'>: {'n': 6, 'nModified': 6, 'ok': 1.0, 'updatedExisting': True}
        # result = cls.update({"yyy": 200}, {"$inc": {"xxx": 1}}, multi=True, check_updated_state=False)
        # 没有则新增
        # update_state = cls.update({"yyy": 203}, {"$inc": {"xxx": 1}}, upsert=True)
            
        # 原子操作
        # 更新,默认没有则新增，upsert=True，new=True返回更新后的文档
        # result = cls.find_and_modify({"yyy": 300}, {"$inc": {"xxx": 1}}, new=True)  #
        # 替换/覆盖/新增
        # result = cls.find_one_and_replace({"yyy": 300}, {"xxx": 1}, return_document=True)
        # result = cls.find_one_and_replace({"yyy": 400}, {"xxx": 1}, sort={"a": -1}, return_document=True)
        # result = cls.find_one_and_replace({"yyy": 500}, {"xxx": 1}, sort=[("a", -1)], return_document=True)
        # 原子更新
        # result = cls.find_one_and_update({"yyy": 400}, {"$inc": {"xxx": 1}}, sort=[("a", 1)], return_document=True)
        # 原子删除
        # result = cls.find_one_and_delete({"yyy": 400}, sort=[("a", 1)])
        # result = cls.find_one_and_delete({"yyy": 400}, sort={"a": -1}, projection={"_id": 0})

        # 过滤
        result = cls.distinct("yyy", {"a": 2})
        result = cls.distinct("_id", {"a": 2})
        result = cls.distinct("yyy", {"a": 2}, **{"maxTimeMS": 5000})

        # 统计result.upserted_id
        count = cls.count()
        count = cls.count({"uid": 102}, skip=10)
        
        # myqsl
        # cls.mysql.insert(cls.collection, [{"name": 2}, {"name": 3}])
        # data = cls.mysql.query_sql(sql="select * from demo where name=(name)", params={"name": 2})
        # data = cls.mysql.query_sql(sql="select * from demo where name=%s", params=[2, ])
        # data = cls.mysql.insert(cls.collection, {"name": 3}, new=True)
        # data = cls.mysql.insert(cls.collection, [{"name": 3}], new=True)
        # datas = cls.mysql.inserts(cls.collection, [{"name": 2}, {"name": 3}], new=True)

        return cls._parse_data(fields)


    @classmethod
    def get_demo_by_spec(cls, spec, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        d = cls.find_one(spec)
        return cls._parse_data(d)


    @classmethod
    def get_demo_list_by_last_id(cls, last_id, count, **kwargs):
        """
        前端列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, last_id = cls.find_list_by_last_id(spec,
                                                      count,
                                                      sort=[("create_at", -1), ],
                                                      fields=False,
                                                      last_id_field=False)
        return [cls._parse_data(d) for d in data_list if d], last_id

    @classmethod
    def get_demo_list_by_page(cls, page, count, **kwargs):
        """
        后台列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, pages = cls.find_list_by_page(spec,
                                                 page,
                                                 count,
                                                 sort=[("create_at", -1), ],
                                                 fields=False)
        return [cls._parse_data(d) for d in data_list if d], pages

    @classmethod
    def create_demo(cls, *args, **kwargs):
        """
        创建
        :return:
        :rtype:
        """
        pass

    @classmethod
    def modify_demo(cls, *args, **kwargs):
        """
        修改
        :return:
        :rtype:
        """
        pass

    @classmethod
    def delete_demo(cls, demo_id, **kwargs):
        """
        删除
        :return:
        :rtype:
        """
        return cls.update({"_id": demo_id}, {"state": -1})

    @classmethod
    def _parse_data(cls, d, **kwargs):
        """
        组装单条数据
        :return:
        :rtype:
        """
        if not d:
            return {}

        d["demo_id"] = str(d.pop("_id", ""))

        return d





from envyaml import EnvYAML
from lcyframe.libs import yaml2py

"""
localhost.yml
env_name: $PYTHONUNBUFFERED
env_name: ${PYTHONUNBUFFERED}
"""
# read file env.yml and parse config
# 支持yml里读取环境变量key: $env
# 读取方式：env["wsgi"]["host"] or env["wsgi.host"]
env = EnvYAML('localhost.yml')
print(env['env_name'])

env2 = yaml2py.load_confog('localhost.yml')
print(env2['env_name'])     # 不支持读取环境变量




# coding=utf-8
import os
import logging
import hmac
import base64
import random
import time
from pathlib import Path
from datetime import timedelta, datetime, date
from string import ascii_letters
import zlib
import hashlib, string
import json
from bson.objectid import ObjectId
import operator
import re
import traceback, ipaddress
import socket, requests
from contextlib import suppress
from urllib.parse import urlparse, urljoin
from requests.utils import requote_uri
try:
    from JWT import JwtToken
    from Rsa import RSA
    from aes import AES
except: pass

_chars = string.printable[:87] + '_' + string.printable[90:95]
to_ObjectId = lambda a: ObjectId(a) if type(a) != ObjectId else a
to_python = lambda s: json.loads(s)
to_json = lambda obj: json.dumps(obj, ensure_ascii=False, sort_keys=True)
fix_path = lambda path: Path(path).as_posix()
traceback = traceback

num_or_alpha = re.compile("^(?!\d+$)[\da-zA-Z_]{5,10}$")  # 仅数字和字母组合，不允许纯数字,长度5~10
startwith_alpha = re.compile("^[a-zA-Z]{5,10}")  # 仅允许以字母开头,长度5~10
lllegal_char = re.compile('^[_a-zA-Z0-9\u4e00-\u9fa5]+$')  # 仅允许中文、英文、数字,_下划线。但不允许非法字符
email_re = re.compile("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$")  # 是否是邮箱
phone_re = re.compile("^(0|86|17951)?(13|14|15|16|17|18)[0-9]{9}$")  # 11位手机号
password_normal = re.compile("^(?:(?=.*)(?=.*[a-z])(?=.*[0-9])).{6,12}$")  # 一般密码 密码必须包含字母，数字,任意字符，长度6~12
password_strong = re.compile("^(?:(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])).{6,12}$")  # 强密码 密码必须包含大小写，数字,任意字符，长度6~12


def random_int(length=6):
    """生成随机的int 长度为length"""
    return ("%0" + str(length) + "d") % random.randint(int("1" + "0" * (length - 1)), int("9" * length))


def gen_random_str(length=6, chars=_chars):
    return ''.join(random.choice(chars) for i in range(length))


def gen_random_sint(length=6):
    """
    获取字符串加数字的随机值
    :param length:
    :return:
    """
    return "".join(random.choice(string.hexdigits) for i in range(length))


def random_string(length=6):
    """生成随机字符串"""
    return ''.join(random.choice(ascii_letters) for i in range(length))


def gen_hmac_key():
    """随机生成长度32位密文"""
    s = str(ObjectId())
    k = gen_random_str()
    key = hmac.HMAC(k, s).hexdigest()
    return key


def enbase64(s):
    """
    编码
    :param s:
    :return:
    """
    if type(s) == bytes:
        return base64.b64encode(s)
    else:
        s = s.encode('utf-8')
        return base64.b64encode(s).decode('utf-8')


def debase64(s):
    """
    解码
    :param s:
    :return:
    """
    bytes_types = (bytes, bytearray)
    return base64.b64decode(s) if isinstance(s, bytes_types) else base64.b64decode(s).decode()


def check_params_date_value(datestr):
    if " " in datestr:
        ymd, hms = datestr.split(" ")
    else:
        ymd, hms = datestr, ""

    seqstr1 = ""
    seqstr2 = ""
    for i in ["-", "/", " ", ":"]:
        if i in ymd:
            seqstr1 = i
        if i in hms:
            seqstr2 = i
    ymd_fmt = []
    hms_fmt = []
    for index, i in enumerate(ymd.split(seqstr1)):
        ymd_fmt.append(["%Y", "%m", "%d"][index])
    for index, i in enumerate(hms.split(seqstr2)):
        hms_fmt.append(["%H", "%M", "%S"][index])
    fmt = ""
    if ymd_fmt:
        fmt += seqstr1.join(ymd_fmt)
    if hms_fmt:
        fmt += " " + seqstr2.join(hms_fmt)
    return string2datetime(datestr, fmt)


class TypeConvert(object):
    """类型转换类, 处理参数"""
    MAP = {int: int,
           float: float,
           list: list,
           bool: bool,
           str: str}

    STR2TYPE = {"int": int,
                "integer": int,
                "string": str,
                "str": str,
                "bool": bool,
                "float": float,
                "list": list,
                "dict": dict,
                "json": json.loads,
                "datetime": check_params_date_value}

    @classmethod
    def apply(cls, obj, raw):
        try:
            tp = type(obj)
            if tp in TypeConvert.MAP:
                return TypeConvert.MAP[tp](raw)
            return obj(raw)
        except Exception as e:
            logging.error("in TypeConvert.apply %s, obj: %s, raw: %s" % (e, obj, raw))
            return None

    @classmethod
    def convert_params(cls, _type, value):
        if _type in ["int", "integer"] and not value:
            value = 0
        try:
            tp = TypeConvert.STR2TYPE[_type]
            return tp(value)
        except Exception as e:
            raise e


def calculate_age(ts):
    """计算年龄"""
    if ts == -1:
        return -1
    born = datetime.fromtimestamp(ts)
    today = date.today()
    return today.year - born.year - ((today.month, today.day) < (born.month, born.day))


def now():
    return int(time.time())


def oid_to_date(oid):
    return int_to_date_string(int(str(oid)[:8], 16))


def int_to_date_string(ts, fm=False):
    fm = fm if fm else "%Y-%m-%d %H:%M"
    try:
        if not ts:
            ts = 0
        return datetime.fromtimestamp(ts).strftime(fm)
    except:
        return datetime.fromtimestamp(time.time()).strftime(fm)


def str2timestamp(date_string, fm=False):
    fm = fm if fm else "%Y-%m-%d"
    return int(time.mktime(time.strptime(date_string, fm)))
    # return int(time.mktime(datetime.strptime(date_string, format).timetuple()))


def timestamp2str(ts, fm=False):
    return int_to_date_string(ts, fm)


def datetime2string(date, fmt="%Y-%m-%d %H:%M:%S"):
    """
     datetime(2020,12,12 12:12:12) ==> 2020-12-12 12:12:12
    """
    return date.strftime(fmt)


def string2datetime(datestr, fmt="%Y-%m-%d %H:%M:%S"):
    """
    datestr: 2022-06-29 14:58:27.047955
    fmt: %Y-%m-%d %H:%M:%S
    2020-12-12 12:12:12 ==> datetime(2020,12,12 12:12:12)

    datestr: 2022-06-29 14:58:27.047955
    fmt: "%Y-%m-%d %H:%M:%S.%f
    datetime(2022-06-29 14:58:27.047955)
    """
    return datetime.strptime(datestr, fmt)


def get_yesterday_midnight():
    # 获取昨天的午夜时间戳
    return get_today_midnight() - 86400


def get_today_midnight():
    # 获取今天的午夜时间戳
    now = int(time.time())
    return now - now % 86400 - 3600 * 8 - 86400


def get_today_lasttime():
    # 获取今天最后一秒时间戳
    now = int(time.time())
    return now - now % 86400 - 3600 * 8 + 24 * 3600 - 1


def get_delta_day(day=0, is_zoo=False, return_fmt="date"):
    """
    获取-n~n天的时间
    datetime.datetime.fromordinal(datetime.datetime.today().toordinal()) + datetime.timedelta(days=-1)
    :param day:
    :param is_zoo: 是否需要0点时间
    :return:

    s1 = get_delta_day(day=1)
    s2 = get_delta_day(return_fmt=int)
    s3 = get_delta_day(return_fmt="string")
    s4 = get_delta_day(return_fmt=datetime)
    s5 = get_delta_day(return_fmt="%Y-%m-%d")
    s6 = get_delta_day(is_zoo=True)
    """

    if is_zoo:
        now = datetime.fromordinal(datetime.today().toordinal())
    else:
        now = datetime.now()

    if day == 0:
        delta_day = now
    else:
        # +-n天后
        delta_day = (now + timedelta(days=int(day)))

    # 返回格式
    if return_fmt == int:
        return int(time.mktime(delta_day.timetuple()))
    elif return_fmt in ["date", datetime]:
        return delta_day
    elif return_fmt in ["string"]:
        return delta_day.strftime("%Y-%m-%d %H:%M:%S")
    else:
        return delta_day.strftime(return_fmt)


def calculate_time_difference(start, end):
    """
    计算2个时间差
    # 示例用法
    start_time = 1635590400  # 时间戳形式的起始时间
    end_time = 16356176800  # 时间戳形式的结束时间
    time_difference = calculate_time_difference(start_time, end_time)
    """

    # 将时间戳转换为datetime对象
    if isinstance(start, int):
        start = datetime.fromtimestamp(start)
    if isinstance(end, int):
        end = datetime.fromtimestamp(end)

    # 计算时间差
    diff = end - start

    days = diff.days
    hours = diff.seconds // 3600
    minutes = (diff.seconds // 60) % 60
    seconds = diff.seconds % 60

    # 返回时间差
    return f"{days}天{hours}时{minutes}分{seconds}秒"

def get_ts_from_object(s):
    if len(s) == 24:
        return int(s[:8], 16)
    return 0


def compress_obj(dict_obj, compress=True):
    """数据压缩"""
    dict_obj = {"$_key_$": dict_obj} if not isinstance(dict_obj, dict) else dict_obj
    if compress:
        return zlib.compress(to_json(dict_obj).encode("utf-8"))
    return to_json(dict_obj)


def uncompress_obj(binary_string, compress=True):
    """数据解压"""
    if compress:
        dict_obj = to_python(zlib.decompress(binary_string).decode("utf-8"))
    else:
        dict_obj = to_python(binary_string)

    if "$_key_$" in dict_obj:
        return dict_obj["$_key_$"]
    else:
        return dict_obj


def get_mod(uid, mod=10):
    return int(uid) % mod


def gen_salt(len=6):
    return ''.join(random.sample(string.ascii_letters + string.digits, len))


def gen_salt_pwd(salt, pwd):
    return hashlib.md5((str(salt) + str(pwd)).encode("utf-8")).hexdigest()


def md5(s):
    s = s if isinstance(s, bytes) else str(s).encode("utf-8")
    return hashlib.md5(s).hexdigest()

def md5_hex(s):
    """
    md5，输出16进制字节格式
    """
    return md5(s)

def md5_bin(s):
    """
    md5，输出二进制字节格式
    """
    s = s if isinstance(s, bytes) else str(s).encode("utf-8")
    return hashlib.md5(s).digest()

def base64_hmac_sha256(key, message):
    key_bytes = key.encode('utf-8')
    message_bytes = message.encode('utf-8')
    hmac_sha256 = hmac.new(key_bytes, message_bytes, hashlib.sha256)
    hmac_sha256_result = hmac_sha256.digest()
    base64_result = base64.b64encode(hmac_sha256_result)
    return base64_result

def pparams(request, params=None):
    print_params = {}
    if params == None:
        params = request.arguments
        params.update(request.body_arguments)
        if hasattr(request, "body_json_arguments"):
            params.update(request.body_json_arguments)

    for k, v in params.items():
        if k not in request.files:
            print_params[k] = v
        else:
            try:
                if "body" in v:
                    tmp = {}
                    tmp.update(v)
                    tmp["body"] = "%d bytes" % len(v["body"])
                    print_params[k] = tmp
                else:
                    print_params[k] = v
            except Exception as e:
                print_params[k] = {}

    return print_params


def version_cmp(version1, version2):
    """比较系统版本号
    v1 > v2 1
    v1 = v2 0
    v1 < v2 -1
    v1: 用户使用的版本
    v2：最新上线的版本
    """

    def normalize(v):
        return [int(x) for x in re.sub(r'(\.0+)*$', '', v).split(".")]

    return operator.gt(normalize(version2), normalize(version1))


def _find_option_with_arg(argv, short_opts=None, long_opts=None):
    """Search argv for options specifying short and longopt alternatives.

    Returns:
        str: value for option found
    Raises:
        KeyError: if option not found.

    Example：
        config_name = _find_option_with_arg(short_opts="-F", long_opts="--config")
    """
    for i, arg in enumerate(argv):
        if arg.startswith('-'):
            if long_opts and arg.startswith('--'):
                name, sep, val = arg.partition('=')
                if name in long_opts:
                    return val if sep else argv[i + 1]
            if short_opts and arg in short_opts:
                return argv[i + 1]
    raise KeyError('|'.join(short_opts or [] + long_opts or []))


def check2json(data):
    if isinstance(data, (list, tuple)):
        for index, item in enumerate(data):
            data[index] = check2json(item)
        return data
    elif isinstance(data, dict):
        for key, value in data.items():
            data[key] = check2json(value)
        return data
    elif isinstance(data, ObjectId):
        return str(data)
    elif isinstance(data, datetime):
        return data.strftime("%Y-%m-%d %H:%M:%S")
    elif data == None:
        return ""
    else:
        return data

def service_kill(name):
    import subprocess
    def cmdprocess(cmdline):
        pipe = subprocess.Popen(cmdline, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
        output, stderr = pipe.communicate()
        return_code = pipe.returncode
        stderr = stderr.decode(errors='replace')
        output = output.decode(errors='replace')
        return output, stderr, return_code
    try:
        cmdline = "ps -ef | grep "+str(name)+" | grep -v grep | awk '{print $2}'"
        output, stderr, return_code = cmdprocess(cmdline)
        if return_code != 0:
            return
        pids = output.splitlines()
        for pid in pids:
            kill_cmdline = f"kill -9 {pid}"
            cmdprocess(kill_cmdline)
    except Exception as e:
        logging.error(f'杀死进程{name}失败:{e}')

def get_ip(address):
    """
    address：www.xxx.com
    返回ip
    """
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
        with suppress(Exception):
            s.connect((address, 1))
            return s.getsockname()[0]
    return None

def ping(address, port):
    test_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    if test_socket.connect_ex((address, port)) == 0:
        return True
    else:
        return False

def check_dns(dns_host):
    """
    检测DNS服务器是否可用
    """
    is_enable = False
    msg = b'\x5c\x6d\x01\x00\x00\x01\x00\x00\x00\x00\x00\x00\x03www\x05baidu\x03com\x00\x00\x01\x00\x01'
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.settimeout(3)
    for i in range(3):
        sock.sendto(msg, (dns_host, 53))
        try:
            sock.recv(4096)
            is_enable = True
            break
        except socket.timeout as e:
            logging.error('{dns} 该dns服务器不可用'.format(dns=dns_host))

    return is_enable

def ip_into_int(ip):
    from functools import reduce
    return reduce(lambda x, y: (x << 8) + y, map(int, ip.split('.')))

def check_internal_ip(ip):
    """
    内网IP 局域网IP
    """
    ip = ip_into_int(ip)
    net_a = ip_into_int('10.255.255.255') >> 24
    net_b = ip_into_int('172.31.255.255') >> 20
    net_c = ip_into_int('192.168.255.255') >> 16
    net_d = ip_into_int('127.0.0.0') >> 24
    net_e = ip_into_int('0.0.0.0') >> 24
    return ip >> 24 == net_a or ip >> 20 == net_b or ip >> 16 == net_c or ip >> 24 == net_d or ip >> 24 == net_e

def check_ssrf_url(url):
    """
    提取有效url，防止SSRF攻击
    URL为内网IP或域名，攻击者通过SSRF漏洞扫描内网漏洞获取权限
    URL中包含端口，攻击者可以扫描发现内网其他机器的服务进行利用
    当请求方法允许其他协议的时候，将可能利用gophar、file等协议进行第三方服务利用，如利用内网的redis获取权限、利用fastcgi进行getshell等
    http://123.123.123.123@10.0.0.1:8888/
    http://10.0.0.1#123.123.123.123
    当request.get(url)时，实际请求10.0.0.1，存在安全隐患
    """
    schema = urlparse(url).scheme or None
    hostname = urlparse(url).hostname or url
    try:
        if not verify_domain(url):
            return ""
        # ip_address = socket.getaddrinfo(hostname, schema)[0][4][0]
        ip_address = ""
        AddressFamily = ""
        for item in socket.getaddrinfo(hostname, schema):
            AddressFamily = item[0]
            if AddressFamily == socket.AddressFamily.AF_INET:
                ip_address = item[4][0]
                break

        if AddressFamily == socket.AddressFamily.AF_INET:
            if check_internal_ip(ip_address):
                return ""
        return (schema or "http") + "://" + hostname
    except Exception as e:
        return ""

def safe_request_url(url, **kwargs):
    """
    安全请求外部url，防止SSRF攻击
    1、通过内网ip攻击
    2、通过30x状态码跳转攻击
    3、通过误导访问不知名url攻击
    """
    def _request_check_location(r, *args, **kwargs):
        if not r.is_redirect:
            return
        url = r.headers['location']

        parsed = urlparse(url)
        url = parsed.geturl()

        # Facilitate relative 'location' headers, as allowed by RFC 7231.
        # (e.g. '/path/to/resource' instead of 'http://domain.tld/path/to/resource')
        # Compliant with RFC3986, we percent encode the url.
        if not parsed.netloc:
            url = urljoin(r.url, requote_uri(url))
        else:
            url = requote_uri(url)

        safe_url = check_ssrf_url(url)
        if not safe_url:
            raise requests.exceptions.InvalidURL("SSRF Attack: %s" % (url, ))

    safe_url = check_ssrf_url(url)
    if not safe_url:
        raise requests.exceptions.InvalidURL("SSRF Attack: %s" % (url,))

    all_hooks = kwargs.get('hooks', dict())
    if 'response' in all_hooks:
        if hasattr(all_hooks['response'], '__call__'):
            r_hooks = [all_hooks['response']]
        else:
            r_hooks = all_hooks['response']

        r_hooks.append(_request_check_location)
    else:
        r_hooks = [_request_check_location]

    all_hooks['response'] = r_hooks
    kwargs['hooks'] = all_hooks
    return requests.get(safe_url, **kwargs)

def verify_username(username, min: int = 4, max: int = 10):
    """
    校验用户名
    :param username: 待验证用户名
    :param min: 最小长度
    :param max: 最大长度
    :return:
    """
    re_str1 = re.compile("^[a-zA-Z](?!\d+$)[\da-zA-Z_]")  # 非数字开头
    re_str2 = re.compile("^(?!\d+$)[\da-zA-Z_]{%d,%d}$" % (min, max))  # 非数字开头，字母与数字组合，长度4~10

    return re.search(re_str1, username) and re.search(re_str2, username)


def verify_nickname(nickname, min: int = 4, max: int = 10):
    """
    校验昵称
    :param nickname: 待验证昵称
    :param min: 最小长度
    :param max: 最大长度
    :return:
    """
    # 允许中文、字母、数字组合，禁止非法字符,长度4~10
    lllegal_char = re.compile('^[_a-zA-Z0-9\u4e00-\u9fa5]{%d,%d}$' % (min, max))
    return re.search(lllegal_char, nickname)


def verify_password(password, min: int = 4, max: int = 10, strong_level=1):
    """
    校验密码
    :param password: 待验证密码
    :param min: 最小长度
    :param max: 最大长度
    :param strong: 密码强度等级
            以下所有规则下的密码都禁止包含空格、表情、汉字
            1、一般密码：符合长度且不能是重复或者连续的数字
            2、正常密码：必须包含字母、数字
            3、强密码：必须包含字母（含大写或小写）、数字
            4、强密码：必须包含字母、数字、特殊英文字符
            5、史上最强密码：必须包含字母（含大小写）、数字、特殊英文字符

    :return:
    """
    password_normal = re.compile(
        "^(?:(?=.*)(?=.*[A-Za-z])(?=.*[0-9])).{%d,%d}$" % (min, max))  # 一般密码 密码必须包含字母(大写或小写)，数字，长度6~12
    password_strong = re.compile(
        "^(?:(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])).{%d,%d}$" % (min, max))  # 强密码 密码必须含字母（含大写），数字，任意字符，长度6~12
    special_char = ["~", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "+", "=", "|", ".", ",", "?", "{", "}",
                    "[", "]", ":"]

    if strong_level == 1:
        if len(password) < min or len(password) > max:
            result = False
        elif is_simple(password) == True:
            result = False
        else:
            result = True
    elif strong_level == 2:
        result = re.search(password_normal, password)
    elif strong_level == 3:
        result = re.search(password_strong, password)
    elif strong_level == 4:
        result = re.search(password_normal, password)
        if result:
            result = any(c in special_char for c in password)
    elif strong_level == 5:
        result = re.search(password_strong, password)
        if result:
            result = any(c in special_char for c in password)
    else:
        result = False

    if not result:
        return False

    # 禁止中文、标签
    if extract_chinese(password):
        return False

    # 禁止非法全角字符
    if extract_illegal_char(password):
        return False

    # 是否含表情
    if password != replace_emoji(password):
        return False

    return True


def verify_phone(phone):
    """
    校验手机号
    :param phone: 待验证phone
    :param min: 最小长度
    :param max: 最大长度

    :return:
    """
    phone_re = re.compile("^(0|86|17951)?(13|14|15|16|17|18)[0-9]{9}$")  # 11位手机号
    return re.search(phone_re, phone)


def verify_email(email, min: int = 7, max: int = 50):
    """
    校验邮箱
    :param phone: 待验证邮箱
    :param min: 最小长度
    :param max: 最大长度

    :return:
    """
    # 是否是邮箱,且长度在7~50之间
    email_re = re.compile("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?){%d,%d}$" % (min, max))
    return re.search(email_re, email)

def verify_email2(email):
    """
    此方法稍慢
    """
    try:
        v = validate_email(email)
        email = v["email"]
        return True
    except EmailNotValidError as e:
        return False

def verify_ip(address):
    """
    校验IP
    支持ipv4\ipv6
    :return:
    # 示例用法
    ipv4 = '192.168.0.1'
    ipv6 = '2001:0db8:85a3:0000:0000:8a2e:0370:7334'

    print(is_valid_ip(ipv4))  # 输出：True
    print(is_valid_ip(ipv6))  # 输出：True
    """
    # 使用正则表达式匹配IPv4和IPv6地址的格式
    pattern = r'^(\d{1,3}\.){3}\d{1,3}$'
    if re.match(pattern, address):
        try:
            # 验证IPv4地址的有效性
            ip = ipaddress.IPv4Address(address)
            return True
        except ValueError:
            return False
    else:
        try:
            # 验证IPv6地址的有效性
            ip = ipaddress.IPv6Address(address)
            return True
        except ValueError:
            return False

def _get_protocol(domain):
    if domain.startswith("https://"):
        protocol = "https://"
    elif domain.startswith("http://"):
        protocol = "http://"
    else:
        protocol = ""
    return protocol

def verify_domain(domain, mtype=1):
    """
    校验域名
    mtype:
        1判断是否为合法域名；
        2提取域名有效部分：https://7w.7.wx.baidu.com >> 7w.7.wx.baidu.com
    :return:
    """
    protocol = _get_protocol(domain)
    domain_re = r'\b((?=[a-z0-9-]{1,63}\.)(xn--)?[a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,63}\b'
    result = re.search(domain_re, domain, re.I)
    if result:
        if mtype == 1:
            return result.group() == domain.replace(protocol, "")
        else:
            return result.group()
    else:
        return False

def extract_domain(domain):
    """
    提取顶级域名
    domain: http://news.baidu.com
    tld = extract_domain(domain)
        >>> ExtractResult(subdomain='news', domain='baidu', suffix='com')
        >>> tld[0] or tld.subdomain == "news"
        >>> tld[1] or tld.domain   == "baidu"
        >>> tld[2] or tld.suffix == "com"
        >>> tld.registered_domain == "baidu.com"

    """

    import tldextract
    tld = tldextract.extract(domain)
    return tld


def extract_domain2(domain):
    """
    提取域名协议，主域，端口,不包含path
    # urllib3.get_host(https://news.baidu.com)  # ('https', 'news.baidu.com', None)
    # urllib3.get_host(news.baidu.com:8888)     # ('http', 'news.baidu.com', 8888)
    # urllib3.get_host(news.baidu.com:9999/abc/123?a=1)     # ('http', 'news.baidu.com', 9999)
    # urllib3.get_host(news.baidu.com/:80:123/abc/123?a=1)     # ('http', 'news.baidu.com', 9999) 注意 path=':80:123/abc/123?a=1'

    protocol:默认http
    """
    import urllib3
    try:
        protocol, host, port = urllib3.get_host(domain)
        return protocol, host, port or ""
    except Exception as e:
        return False


def extract_chinese(str):
    """
    提取中文
    :param str:
    :return:
    """

    regex_str = ".*?([\u4E00-\u9FA5]+).*?"
    match_obj = re.findall(regex_str, str)
    return match_obj


def extract_illegal_char(str):
    """
    提取非法半角字符，空格
    :param str:
    :return:
    """
    regex_str = "([！￥……（）——，。、？；：【】「」《》“”‘'  ]+).*?"
    match_obj = re.findall(regex_str, str)
    return match_obj


def extract_illegal_char2(str):
    """
    性能较差
    判断是否含有特殊非法字符
    :param str:
    :return:
    """
    # 所有非法字符
    all_illegal_char = range(0x0, 0x10ffff)  # 总数110万个
    for s in str:
        if s in all_illegal_char:  # 判断一个字符大约需要20ms
            return False
    return True


def change_emoji(char_str):
    """
    将表情转为字符
    :param str: 123🆚456
    :param result: 123:VS_button:456

    :param str: 表情🏷
    :param result: 表情:label:

    :return:
    """
    import emoji
    return emoji.demojize(char_str)


def replace_emoji(char_str, rep_str=''):
    """
    将字符串中的表情替换为指定的字符，默认替换为空，即过滤表情
    常用语用户名、昵称注册等禁止表情的输入
    :param char_str:
    :param rep_str:
    :return:
    """
    try:
        co = re.compile(u'[\U00010000-\U0010ffff]')
    except re.error:
        co = re.compile(u'[\uD800-\uDBFF][\uDC00-\uDFFF]')
    return co.sub(rep_str, char_str)


def is_simple(password):
    """
    判断字符串密码是否为简单重复或者纯数字或连续的数字
    若是，返回True
    若不是，返回False
    :param password:
    :return:
    """
    if password.isdigit():
        password = [int(n) for n in password]
        if len(set(password)) == 1:
            return True

        for index, a in enumerate(password[: -1]):
            b = password[index + 1]
            if (b - a) != 1:
                return False
        return True
    else:
        password = [n for n in password]
        if len(set(password)) == 1:
            return True
        else:
            return False

def is_letter(char_str):
    """
    字符串是否全部由字母组成
    str.isalpha(): abc、a是bc都返回真，不满足
    """
    return re.match(re.compile(r"[a-zA-Z]+$"), char_str)

def transfer_str(str):
    """
    mongo 正则匹配 转义
    :param str:
    :return:
    """
    new_str = ""
    special = ['/', '^', '$', '*', '+', '?', '.', '(', ')']
    for c in str:
        if c in special:
            new_str += '\\'
        new_str += c
    return new_str


def supper_format(filename, format_list):
    """
    判断文件是否在指定的格式内
    :param filename: abc.e.f.tar.gz
    :param format_list: ["rar", "tar", "tar.gz"]
    :return: True
    """
    return any(filter(lambda x: filename.lower().endswith(x.lower()), format_list))


def get_filename_format(filename, format_list):
    """
    截取文件名和格式名
    针对压缩包名称较为复杂的情况
    :param filename: abc.e.f.tar.gz
    :param format_list: ["rar", "tar", "tar.gz"]
    :return: abc.e.f
    """
    if "." not in filename:
        return filename, ""

    exists_format = False

    for i in format_list:
        if not i.startswith("."):
            suffix = "." + i
        else:
            suffix = i
        if filename.lower().endswith(i):
            name, format = filename.rsplit(suffix, 1)[0], i
            exists_format = True
            break

    if exists_format:
        return name, format
    else:
        return filename.rsplit(".", 1)


def auto_rename(name, uncompress_path, n=1):
    """
    指定一个解压位置，若有同名文件夹存在，则自动重命名
    :param name:
    :param uncompress_path:
    :param n:
    :return:
    """
    while n < 100:
        if n > 1:
            dst_path = os.path.join(uncompress_path, name + "-" + str(n))
        else:
            dst_path = os.path.join(uncompress_path, name)

        if os.path.exists(dst_path):
            n += 1
            return auto_rename(name, uncompress_path, n)
        else:
            if n > 1:
                return name + "-" + str(n), dst_path
            else:
                return name, dst_path
    raise


def undecompress(compress_path, uncompress_path):
    """
    解压包
    :param compress_path: 压缩包目前所在目录: /code/123.zip
    :param uncompress_path: 需要解压至该目录:/code/newname
    :param decompress_format: 支持的解压格式列表["zip", "rar", "7z", "tar", "tbz2", "tgz", "tar.bz2", "tar.gz", "tar.xz", "tar.Z"]
    :return:
    """
    import patoolib
    try:
        if not os.path.exists(uncompress_path): os.makedirs(uncompress_path)
        patoolib.extract_archive(compress_path, outdir=uncompress_path)
    except Exception as e:
        os.remove(uncompress_path)
        logging.error(str(e))
    else:
        return True


def pdf2img(pdf_path, to_path):
    '''
    # 将PDF转化为图片
    pdf_path pdf文件的路径
    to_path 保存文件夹目录
    zoom_x x方向的缩放系数
    zoom_y y方向的缩放系数
    rotation_angle 旋转角度
    '''
    try:
        import fitz
        doc = fitz.open(pdf_path)
        print("PDF转图片，任务文件：%s" % pdf_path)

        ts = now()
        for pg in range(doc.pageCount):
            print("\r共%s页,正在转换第%s/%s张" % (doc.pageCount, pg + 1, doc.pageCount), end="")
            page = doc[pg]
            rotate = int(0)
            # 每个尺寸的缩放系数为8，这将为我们生成分辨率提高64倍的图像。
            zoom_x = 6.0
            zoom_y = 6.0
            trans = fitz.Matrix(zoom_x, zoom_y).prerotate(rotate)
            pm = page.get_pixmap(matrix=trans, alpha=False)
            save_name = '{:01}.png'.format(pg + 1)
            pm.save(os.path.join(to_path, save_name))
        print()
        print("耗时:%s秒" % str(now() - ts))
    except Exception as e:
        print(str(e))

def imagecode(stype=4, args_value=None, return_type=2, filename=None, **kwargs):
    """
    生成图形验证码
    stype: 验证码类型
    args_value： 指定生成参数；值可以是：指定的验证码，生成的长度；加减法符号
    return_type： 返回类型 1 返回字节码；2返回base64；3保存到指定文件
    filename： 保存路径/名称.png
    kwargs：生成参数
    dict(
        font_color_values=font_color_values,
        font_background_value='#87CEFF',
        draw_dots=True,
        dots_width=5,
        dots_count=200,
        draw_lines=True,
        lines_width=4,
        lines_count=10,
        lines_background_value="#ffffff",
    )

    例子：
    print(imagecode(1, "123456", return_type=2))
    print(imagecode(2, 6, return_type=2))
    print(imagecode(3, 6, return_type=3))
    print(imagecode(4, 6, return_type=3))
    print(imagecode(5, "+", return_type=3))

    """
    from .gvcode import VFCode
    default_kw = dict(
        font_background_value='#87CEFF',
        draw_dots=True,
        dots_width=5,
        dots_count=200,
        draw_lines=True,
        lines_width=4,
        lines_count=10,
        lines_background_value="#ffffff",
    )
    if kwargs:
        default_kw.update(kwargs)
    vc = VFCode(**default_kw)

    mp = {
        # 自定义验证码
        # vc.generate('abcd')
        1: vc.generate,

        # 数字验证码（默认5位）
        # vc.generate_digit()
        # vc.generate_digit(4)
        2: vc.generate_digit,

        # 字母验证码（默认5位）
        # vc.generate_alpha()
        # vc.generate_alpha(5)
        3: vc.generate_alpha,

        # 数字字母混合验证码（默认5位）
        # vc.generate_mix()
        # vc.generate_mix(6)
        4: vc.generate_mix,

        # 数字加减验证码（默认加法）
        # 数字加减验证码（加法）
        # vc.generate_op('+')
        # 数字加减验证码（减法）
        # vc.generate_op('-')
        # 数字乘法，字母x
        # vc.generate_op('x')
        5: vc.generate_op,
    }
    if stype not in [1, 2, 3, 4, 5]:
        raise Exception("类型错误")

    if args_value is not None:
        mp[stype](args_value)
    else:
        mp[stype]()

    code = vc.code
    if return_type == 1:    # 返回字节码
        code_data = vc.get_img_bytes()
    elif return_type == 2:
        code, code_data = vc.get_img_base64()
    else:
        vc.save(filename)
        code_data = filename

    return code, code_data

def get_string_size(string, encode_type="utf-8"):
    """
    获取字符串字节数
    """
    if isinstance(string, bytes):
        string = string.decode()
    return len(string.encode(encode_type))


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import base64
import rsa as _rsa


class RSA(object):
    """
    # 公钥加密
    # str = rsa.encrypt(self.params, publicKey)

    # 私钥解密
    # str = rsa.decrypt(self.params, privateKey).decode()

    # 私钥签名
    # signature = rsa.sign(message, privkey, 'SHA-1')

    # 公钥验证
    s = rsa.verify(message, sign, public_key)

    # =================================
    # 场景〇：密钥保存导入
    # =================================
    # 保存密钥
    with open('public.pem','w+', encoding='utf-8') as f:
        print(pubkey.save_pkcs1().decode())
        f.write(pubkey.save_pkcs1().decode())

    with open('private.pem','w+') as f:
        print(privkey.save_pkcs1().decode())
        f.write(privkey.save_pkcs1().decode())

    # 导入密钥
    with open('public.pem','r') as f:
        pubkey = rsa.PublicKey.load_pkcs1(f.read().encode())

    with open('private.pem','r') as f:
        privkey = rsa.PrivateKey.load_pkcs1(f.read().encode())

    """

    @classmethod
    def gen_rsa_key(cls, nbits=1024):
        """
        生成key
        :param nbits:
        :return:
        """
        cls.pubkey, cls.privkey = _rsa.newkeys(int(nbits))
        return cls.save_pkcs(cls.pubkey), cls.save_pkcs(cls.privkey)

    @classmethod
    def encode(cls, str, pubkey):
        """
        公钥加密
        :param str:
        :param pubkey:
        :return:
        """
        str = str.encode() if not isinstance(str, bytes) else str
        return base64.b64encode(_rsa.encrypt(str, cls.load_pkcs(pubkey, "pub"))).decode()

    @classmethod
    def decode(cls, str, privkey):
        """
        公钥加密
        :param str:
        :param pubkey:
        :return:
        """
        str.encode() if not isinstance(str, bytes) else str
        str = base64.b64decode(str)
        return _rsa.decrypt(str, cls.load_pkcs(privkey, "pri")).decode()

    @classmethod
    def sign(cls, str, privkey, hash_method="MD5"):
        """
        私钥签名
        :param str:
        :param privkey:
        :param Use 'MD5', 'SHA-1', 'SHA-224', SHA-256', 'SHA-384' or 'SHA-512'.
        :return:
        """
        str = str.encode() if not isinstance(str, bytes) else str
        return base64.b64encode(_rsa.sign(str, cls.load_pkcs(privkey, "pri"), hash_method)).decode()

    @classmethod
    def verify(cls, str, sign, pubkey):
        """
        公钥验签
        :param str:
        :param pubkey:
        :return:
        """
        str = str.encode() if not isinstance(str, bytes) else str
        try:
            sign = base64.b64decode(sign)
            result = _rsa.verify(str, sign, cls.load_pkcs(pubkey, "pub"))
        except Exception as e:
            return False
        else:
            return True


    @classmethod
    def load_pkcs(cls, str, t):
        """
        转为rsa对象
        :param key:
        :return:
        """
        if t == "pub":
            return _rsa.PublicKey.load_pkcs1(str)
        else:
            return _rsa.PrivateKey.load_pkcs1(str)

    @classmethod
    def save_pkcs(cls, key):
        """
        rsa对象转为字节码，便于存储、阅读
        转为rsa对象
        :param key:
        :return:
        """

        return key.save_pkcs1()


    @classmethod
    def save_to_file(cls, path=None):
        """
        生成公钥私钥并保存
        """
        import os
        pubkey, privkey = r.gen_rsa_key()
        p = os.path.join(path, "privkey.pem") if path else "privkey.pem"
        with open(p, "wb") as x:  # 保存私钥
            x.write(privkey)

        p = os.path.join(path, "pubkey.pem") if path else "pubkey.pem"
        with open(p, "wb") as x:  # 保存公钥
            x.write(pubkey)

    @classmethod
    def read_from_file(cls, filename, t):
        """
        读取公钥私钥
        """
        with open(filename, "rb") as x:
            str = x.read()

        return cls.save_pkcs(cls.load_pkcs(str, t))

if __name__ == '__main__':
    r = RSA()
    str = """$#5qead"""

    pubkey, privkey = r.gen_rsa_key()

    # 加密
    encode_str = r.encode(str, pubkey)
    print("加密：", encode_str)
    # 解密
    decode_str = r.decode(encode_str, privkey)
    print("解密：", decode_str)
    print("结果", decode_str == str)


    # 签名
    s = r.sign(str, privkey, "SHA-512")
    print("签名", s)

    # 验签
    result = r.verify(str, s, pubkey)
    print("验签", result)

    # 生成公钥私钥
    # r.save_to_file()


# coding=utf-8
import os, re
import logging
import random
import time
from datetime import datetime, date
import zlib, hmac
from bson import BSON, ObjectId
from functools import wraps
from lcyframe.libs import utils, funts, cprint, JWT, aes, Rsa, errors
from utils import errors, keys


def get_excel_url(host, img, path=None):
    """excel"""
    if path is None:
        path = "/data/excel"

    if "http" in host and host.startswith("http"):
        pass
    else:
        host = "http://" + host

    p = host + path
    p = os.path.join(p, img)
    return p


def get_imge_url(host, img, path=None):
    """******"""
    if path is None:
        path = "/data"

    if "http" in host and host.startswith("http"):
        pass
    else:
        host = "http://" + host

    p = os.path.join(host, path, img)
    return p


def save_filedata(data, base_path, **kwargs):
    """保存文件"""
    if not data:
        return "", ""

    filename = gen_filename(data, **kwargs)
    save_path = gen_filepath(base_path, filename)

    if not os.path.exists(base_path):
        os.mkdir(base_path)

    if not os.path.exists(save_path):
        with open(save_path, "wb") as f:
            body = data["body"] if isinstance(data["body"], bytes) else data["body"].encode("u8")
            f.write(body)

    return filename, save_path


def gen_filepath(base_path, filename=None):
    if not os.path.exists("%s" % base_path):
        os.makedirs("%s" % base_path)

    save_path = "%s/%s" % (base_path, filename)
    return save_path


def gen_filename(file_data, prefix="", suffix="", name_type="unique", require_format=[], require_size=0):
    """
    生成文件名
    根据数据md5对文件去重
    :param body:
    :param name_type: unique名称按数据去重，date 按时间格式命名 random 随机命名
    :return:

    """

    metadata = get_fileinfo(file_data, require_format=require_format, require_size=require_size)

    if name_type == "unique":
        filename = metadata["unique_name"]
    elif name_type == "date":
        filename = "%s-%s" % (datetime.now().strftime("%Y%m%d%H%M%S"), str(utils.random_int(4)))
    elif name_type == "random":
        filename = str(utils.gen_random_sint(16))
    else:
        filename = metadata["name"]

    if metadata["format"]:
        return "%s%s%s.%s" % (prefix, filename, suffix, metadata["format"])
    else:
        return "%s%s%s" % (prefix, filename, suffix)


def get_fileinfo(file_data=None, file_path=None, require_format=[], require_size=0):
    """
    读取文件基本信息
    :param file_data:
    :return:
    """
    if file_data:
        if not isinstance(file_data, dict):
            file_data = {
                "body": file_data,
                "filename": "default"
            }
        body = file_data["body"]
        filename = file_data["filename"]
        unique_name = utils.md5(body if isinstance(body, bytes) else body.encode("u8"))
        name, format = file_data["filename"].rsplit(".", 1) if "." in file_data["filename"] else (file_data["filename"], "")
        # TODO Fix 包含有以下几种符号的文件名，会导致系统解压命令失败：'('、')'。如需在线调用系统命令解压，请打开注释.
        #  推荐使用python的Patool库进行统一加解压，可避免此问题
        # if format in ["rar", "zip", "pkg", "gz", "tar", "jar", "7z"]:
        #     if name.find("(") or name.find(")"):
        #         name = name.replace("(", "（").replace(")", "）")
        #
        #     filename = name + "." + format if format else name

        size = len(body)
    elif file_path and os.path.exists(file_path):
        with open(file_path, "r") as p:
            body = p.read()
            unique_name = utils.md5(body)
            name, format = file_data.split("/")[-1].split(".")
            size = len(body)
    else:
        raise

    if require_format and format.lower() not in require_format:
        raise errors.UploadError("请上传指定格式：" + ",".join(require_format))
    if require_size and size > require_size:
        raise errors.UploadError("大小不能超过%sM" % int(require_size/1024/1024))

    return {
        "filename": filename,
        "name": name,
        "format": format.lower(),
        "size": size,
        "unique_name": unique_name,
        "unique_filename": ("%s.%s" % (unique_name, format)) if format else unique_name
    }


def check__forestall_brute(app, key):
    """
    防止暴力破解密码登录
    登录前判断
    """
    if not hasattr(app, "forestall_brute_mapping"):
        return True
    else:
        logs = app.forestall_brute_mapping.get(key, {"count": 0, "ts": 0})
        if logs and logs["count"] >= 10 and logs.get("expire") > utils.now():
            raise errors.ErrorInvalid("你的操作已被系统判定为攻击行为")
        else:
            return True

def add_forestall_brute(app, key):
    """
    防止暴力破解密码登录
    当密码错误后，将key加入规则
    如次数得到指定值，将限定一段时间内不能登录
    """
    if not hasattr(app, "forestall_brute_mapping"):
        setattr(app, "forestall_brute_mapping", {})
    logs = app.forestall_brute_mapping.get(key, {"count": 0, "ts": 0})
    logs["count"] += 1
    logs["ts"] = utils.now()
    if logs["count"] >= 10:
        logs["expire"] = utils.now() + 15*60
    app.forestall_brute_mapping[key] = logs

def remove_forestall_brute(app, key):
    """
    防止暴力破解密码登录,以下操作后，移除key
    登陆成功后
    修改密码后
    """
    if hasattr(app, "forestall_brute_mapping"):
        app.forestall_brute_mapping.pop(key, False)

def get_excel_path(root_path, file_name=None):
    if not file_name:
        file_name = "%s.xlsx" % (datetime.now().strftime("%Y%m%d%H%M%S") + str(random.randint(0, 99)))

    path = datetime.now().strftime("%Y%m%d")

    base_path = 'data/excel'
    if not os.path.exists("%s/%s/%s" % (root_path, base_path, path)):
        os.makedirs("%s/%s/%s" % (root_path, base_path, path))

    save_path = "%s/%s/%s/%s" % (root_path, base_path, path, file_name)

    file_name = '%s/%s' % (path, file_name)

    return save_path, file_name


def get_now_time_tuple(ts):
    d = utils.int_to_date_string(ts, fm="%d")
    m = utils.int_to_date_string(ts, fm="%m")
    y = utils.int_to_date_string(ts, fm="%Y")
    return y, m, d


def pc_or_mobile(ua):
    """
    判断客户端PC还是手机端
    :param ua: ******User-Agent******
    :return:
    """
    factor = ua
    is_mobile = False
    _long_matches = r'googlebot-mobile|android|avantgo|blackberry|blazer|elaine|hiptop|ip(hone|od)|kindle|midp|mmp' \
                    r'|mobile|o2|opera mini|palm( os)?|pda|plucker|pocket|psp|smartphone|symbian|treo|up\.(browser|link)' \
                    r'|vodafone|wap|windows ce; (iemobile|ppc)|xiino|maemo|fennec'
    _long_matches = re.compile(_long_matches, re.IGNORECASE)
    _short_matches = r'1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)' \
                     r'|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)' \
                     r'|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw' \
                     r'|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8' \
                     r'|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit' \
                     r'|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)' \
                     r'|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji' \
                     r'|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|e\-|e\/|\-[a-w])|libw|lynx' \
                     r'|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(di|rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi' \
                     r'|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)' \
                     r'|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg' \
                     r'|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21' \
                     r'|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-' \
                     r'|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it' \
                     r'|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)' \
                     r'|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)' \
                     r'|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit' \
                     r'|wi(g |nc|nw)|wmlb|wonu|x700|xda(\-|2|g)|yas\-|your|zeto|zte\-'

    _short_matches = re.compile(_short_matches, re.IGNORECASE)

    if _long_matches.search(factor) != None:
        is_mobile = True
    user_agent = factor[0:4]
    if _short_matches.search(user_agent) != None:
        is_mobile = True

    return is_mobile

def tts():
    return time.time() * 1000

def get_argument(self):
    """
    ******
    :param self:
    :return:
    """
    argument = {}
    try:
        keys = self.params.keys()
        for key in keys:
            if key == "secret_key":
                continue
            argument[key] = self.get_argument(key)
    except:
        argument = {k: v[0] for k, v in self.request.body_arguments.items() or self.request.arguments.items()}

    return argument


def admin(permission_bit=0):
    """
    认证：支持多个客户端同时访问
    1、验证是否登录：token
    2、验证是否合法请求：sign
    3、验证是否来自管理端请求（可指定其他端）
    :param permission_bit: 0 游客可访问
    :return:

    :::
    :::密码的加密流程：登录时需提交密码的密文格式
    1、前/后端均采用AES加密，输出十六进制字符串,参数详细如下（密钥写死在客户端）
        key: 82b0bfa746bdb72e310609070defa1cd
        iv: 3eddc41dd41239c8
        mode: ECB
        padding: zeropadding
    2、验签样例：
        密码明文：123456
        AES密文：016078ca0cbf7fa1655bc609418c576d

    ::: Token的生成流程：（登录后获得）
    1、后端讲提交的密码进行解密，得到明文，在通过md5(slat+password)与数据库进行对比
    2、如密码正确，则登陆成功，服务端将uid和当前时间戳组成dict={"uid": 100,"ts": 1636349973}
    3、用相同的秘钥（key=82b0bfa746bdb72e310609070defa1cd）通过JWT加密得到token，写入redis并返回给客户端
    4、客户端把uid和token放置在header里，发起后续请求

    Token的验证样例：
    加密前：{"uid": 100,"ts": 1636349973}
    JWT加密后："eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJtZXNzYWdlIjp7InRzIjoxNjM2MzQ5OTczLCJ1aWQiOjEwMH0sImV4cCI6MTYzNjM1MzU3M30.h_eHntkiaTM9P4IQ_tDKuWo8ijdtH1Cwff9aM_3-N_M"

    ::: Sign数字签名生成流程（前端生成、后端验证。调用接口必须提供）
    1、客户端把uid和token放置在header里，每次调用接口时，服务端将在redis内重置该Token的有效期，如果key不存在，则提示客户端重新登陆
    2、在请求参数里新增timestamp、nonce参数；前者是当前请求的时间戳，后者是一个随机参数；
    3、利用&拼接在请求参数query_string里，服务端判断当前时间与提交的timestamp是否相差在1分钟内的接口，若是，正常请求，否则返回错误，防止重放攻击，防刷
    4、服务端每次收到请求后，都需要保存nonce到redis，过期时间1分钟，如果遇到重复的nonce，视为重复请求，返回错误，保证接口冥等性
    5、客户端把所有请求参数名按照首字母大到小（小写）方式进行排序（值为空的参数去掉），利用&拼接，得到待签名字符串str1
    6、将str1进行base64编码得到str2
    7、将token和str2拼接（token+str2），得到str3
    8、将str3进行md5加密（md5(str3)），输出十六进制字符串，得到sign
    9、验证：
        待加密参数：{"a": 1, "b": 2, "timestamp": 1634094357, "nonce": "tfbksz"}
        # p1 利用&拼接
        str1 = 'a=1&b=2&nonce=tfbksz&timestamp=1634094357'
        # p2 base64编码
        str2 = 'YT0xJmI9MiZub25jZT10ZmJrc3omdGltZXN0YW1wPTE2MzQwOTQzNTc='
        # p3 拼接token
        str3 = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1aWQiOjEsImV4cCI6MTYzNDA5OTAxNn0.djsmWDtwBR2AGGF79HufFcN1ofD5NCW0QI9H59EMB-0YT0xJmI9MiZub25jZT10ZmJrc3omdGltZXN0YW1wPTE2MzQwOTQzNTc='
        # p4 md5加密 输出32位小写字符串
        sign = '990b897e66a59a976498ad0a5e29d9fd'

    """

    def wrap(method):
        @wraps(method)
        def has_role(self, *args, **kwargs):
            self.user_id = self.request.headers.get("uid", 0)
            if type(self.user_id) != int and self.user_id.isdigit():
                self.user_id = int(self.user_id)
            self.token = self.request.headers.get("token", "")
            if permission_bit:
                # 是否合法访问
                _check_token(self, permission_bit)

                # sign
                # _verifi_sign(self, self.token)

                # 只有超管账号允许访问系统端
                _check_admin(self)

            self.params = funts.get_params(self)

            # 对已登录的用户进行权限拦截
            _check_permissions(self, permission_bit)

            if self.app_config.get("logging_config", {}).get("level", "debug") == "debug":
                logging.debug(
                    "请求参数: " + (utils.to_json(utils.check2json(utils.pparams(self.request, params=self.params)))
                                if hasattr(self, "params") else str(utils.pparams(self.request) or {})))

            return method(self, *args, **kwargs)

        return has_role

    return wrap


def _check_token(self, permission_bit):
    token_debug = self.token_config.get("debug", False)
    if not token_debug:
        if permission_bit != 0:
            if not self.user_id or not self.token:
                raise errors.ErrorTokenInvalid

            jwt = JWT.JwtToken(self.token_config["secret"], self.token_config["expire"])
            try:
                if jwt.is_expire(self.token):
                    raise errors.ErrorTokenExpireInvalid

                if not jwt.is_validate(self.token):
                    raise errors.ErrorTokenInvalid

                payload = jwt.decode(self.token)
                if payload and payload["user_id"] != self.user_id:
                    raise errors.ErrorTokenInvalid

                self.set_header("Token", jwt.refresh_token(self.token))
            except Exception as e:
                raise errors.ErrorTokenInvalid


def _verifi_sign(self, token):
    """
    验签
    :param self:
    :param token:
    :param secret_key:
    :return:
    """

    def source_argument():
        """
        获取原始参数值
        """
        argument = {}
        try:
            keys = self.params.keys()
            for key in keys:
                if key == "secret_key":
                    continue
                argument[key] = self.get_argument(key)
        except:
            argument = {k: v[0] for k, v in self.request.body_arguments.items() or self.request.arguments.items()}

        return argument

    argument = source_argument()
    argument["timestamp"] = int(self.get_argument("timestamp", 0))
    argument["nonce"] = self.get_argument("nonce", "")
    sign = self.get_argument("sign", "")

    if not argument["timestamp"] or not argument["nonce"]:
        raise errors.ErrorMissingArgument("Missing timestamp, nonce")

    new_sign = gen_sign(token, **argument)

    if new_sign != sign:
        raise errors.ErrorTokenInvalid

    # ExpireQuery
    expire_query = int(self.token_config.get("expire_query", 0))
    if expire_query != 0:
        if utils.now() - argument["timestamp"] > expire_query:
            raise errors.ErrorExpireQuery

    # DuplicateQuery
    duplicate_query = int(self.token_config.get("duplicate_query", 0))
    if duplicate_query != 0:
        if self.redis.exists(keys.NONCE_MARK % argument["nonce"]):
            raise errors.ErrorDuplicateQuery

        self.redis.setex(keys.NONCE_MARK % argument["nonce"], duplicate_query, 1)


def _check_admin(self):
    self.is_admin = True

    self.member = self.model.AdminModel.get_admin(self.user_id)
    if not self.member:
        raise errors.ErrorUserNotFound
    if self.member["company_id"] != 1:
        raise errors.ErrorNoRolePermission("无权访问")

    # self.gid = self.member["gid"]


def _check_permissions(self, permission_bit):
    """
    验证权限
    """
    if permission_bit and self.request.path not in ["/permission/enable"]:
        has_perm, msg = self.model.RBACPermissionModel.has_permission(self)
        if not has_perm:
            raise errors.ErrorNoApiPermission


def verifi_ip(self, ip_list):
    """
    验证白名单
    :param self:
    :param ip_list:
    :return:
    """
    if not self.application.app_config["security"]["verifi_ip"]:
        return

    ip_list = [ip.encode("u8") for ip in ip_list]
    remote_ip = self.request.remote_ip.encode("u8")
    s = "*" in ip_list or remote_ip in ip_list
    if not s:
        raise errors.ErrorTranIpError


def gen_sign(token, **kwargs):
    """
    验签 3.0
    :param sign:
    :param kwargs:
    :return:
    """
    str1 = gen_sign_str(kwargs)
    str2 = utils.enbase64(str1)
    sign = utils.gen_salt_pwd(token, str2)
    return sign


def gen_sign_str(params):
    """
    ******
    :param params:
    :return:
    """
    keys = [k for k, v in params.items() if v != "" and k != "sign"]
    keys.sort()
    sign_str = "&".join(["%s=%s" % (k, str(params[k])) for k in keys])

    return sign_str

def sort_query_string(query_string):
    """
    参数按照key小写，从小到大排序；并用&拼接；
    注意：不适用于过滤value=空字符串的情况

    """
    t = query_string.split("&")
    t.sort()
    return "&".join(t)


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import random, string
from test_script import *
from lcyframe.libs import cprint, utils


def get_permission_groups(*args, **kwargs):
    """
    角色详情:

    参数:
    - group_id # 必填,int,角色id
    """
    headers = {}
    files = {}
    params = {
        "group_id": 27,
        "content_type": 0
    }
    return send(methed="get", url="/permission/groups", params=params, headers=headers, files=files)


def post_permission_groups(*args, **kwargs):
    """
    新增角色:

    参数:
    - name # 必填,str,权限名称
    - content_type # 必填,int,0系统端角色，1业务的角色
    - describes # 选填,str,角色说明
    """
    headers = {}
    files = {}
    params = {
        "name": "权限名称",
        "content_type": 1,
        "describes": "角色说明",
    }
    return send(methed="post", url="/permission/groups", params=params, headers=headers, files=files)


def put_permission_groups(*args, **kwargs):
    """
    修改角色:

    参数:
    - group_id # 必填,int,id
    - name # 选填,str,权限名称
    - state # 选填,int,1启用，0禁用
    - describes # 选填,str,角色说明
    """
    headers = {}
    files = {}
    params = {
        "group_id": 48,
        "name": "角色",
        # "state": 0,
        "describes": "角s色说ss明",
        # "content_type": 1,
    }
    return send(methed="put", url="/permission/groups", params=params, headers=headers, files=files)


def delete_permission_groups(*args, **kwargs):
    """
    删除角色:

    参数:
    - group_id # 必填,int,id
    """
    headers = {}
    files = {}
    params = {
        "group_id": "(必填)id",
    }
    return send(methed="delete", url="/permission/groups", params=params, headers=headers, files=files)


def get_permission_groups_list(*args, **kwargs):
    """
    角色列表:

    参数:
    - page # 选填,int,翻页码
    - count # 选填,int,每页显示条数
    - state # 选填,int,状态
    - content_type # 必填,int,0系统端角色，1业务的角色
    - company_id # 必填,int,指定商户，默认所有角色。
    """
    headers = {}
    files = {}
    params = {
        "page": 1,
        "count": 10,
        # "state": 1,
        "content_type": 1,
    }
    return send(methed="get", url="/permission/groups/list", params=params, headers=headers, files=files)


def get_permission_groups_members(*args, **kwargs):
    """
    该角色的用户
    参数:
    - group_id # int
    """
    headers = {}
    files = {}
    params = {
        # "company_id": 1,
        "group_id": 45
    }
    return send(methed="get", url="/permission/groups/members", params=params, headers=headers, files=files)


def post_permission_groups_members(*args, **kwargs):
    """
    该角色的用户
    参数:
    - group_id # int
    """
    headers = {}
    files = {}
    params = {
        "group_id": 45,
        "company_id": 14,
        "user_id": "53,55"
    }
    return send(methed="post", url="/permission/groups/members", params=params, headers=headers, files=files)


def post_permission_groups_check(*args, **kwargs):
    """
    编辑角色权限:权限勾选或取消,修改成功后，建议刷新本地权限缓存

    参数:
    - group_id # 必填,int,角色id
    - key # 必填,str,接口标识符
    - method # 必填,str,接口方法
    - value # 必填,int,1勾选，0取消
    """
    headers = {}
    files = {}
    params = {
        "group_id": "(必填)角色id",
        "key": "(必填)接口标识符",
        "method": "(必填)接口方法",
        "value": "(必填)1勾选，0取消",
    }
    return send(methed="post", url="/permission/groups/check", params=params, headers=headers, files=files)


def post_permission_groups_checks(*args, **kwargs):
    """
    批量编辑角色权限:权限勾选或取消,修改成功后，建议刷新本地权限缓存

    参数:
    - group_id # 必填,int,角色id
    - key # 必填,str,接口标识符
    - method # 必填,str,接口方法
    - value # 必填,int,1勾选，0取消
    """
    headers = {}
    files = {}
    params = {
        "group_id": 48,
        "permissions": utils.to_json({
            "demo4-api:get": 1
        })
    }
    return send(methed="post", url="/permission/groups/checks", params=params, headers=headers, files=files)


def get_permission_groups_company(*args, **kwargs):
    """
    商户的所有角色:查看某商户的所有角色，不分页

    参数:
    - company_id # 必填,int,商户id
    """
    headers = {}
    files = {}
    params = {
        "company_id": 19,
        "content_type": 1,
    }
    return send(methed="get", url="/permission/groups/company", params=params, headers=headers, files=files)


def post_permission_groups_company(*args, **kwargs):
    """
    分配角色:把角色分配给商户

    参数:
    - group_id # 必填,str,角色id， 多个角色用英文逗号隔开
    - company_id # 必填,int,商户id
    """
    headers = {}
    files = {}
    params = {
        "group_id": "31,32,33",
        "company_id": 19,
    }
    return send(methed="post", url="/permission/groups/company", params=params, headers=headers, files=files)


def delete_permission_groups_company(*args, **kwargs):
    """
    移除角色:将角色从商户下移除，属于该角色的用户将失去该角色的所有权限

    参数:
    - group_id # 必填,int,角色id
    - company_id # 必填,int,商户id
    """
    headers = {}
    files = {}
    params = {
        "group_id": "34",
        "company_id": 10,
    }
    return send(methed="delete", url="/permission/groups/company", params=params, headers=headers, files=files)


def get_permission(*args, **kwargs):
    """
    权限详情:

    参数:
    - permission_id # 必填,int,id
    """
    headers = {}
    files = {}
    params = {
        "permission_id": 277,
    }
    return send(methed="get", url="/permission", params=params, headers=headers, files=files)


def post_permission(*args, **kwargs):
    """
    新增:

    参数:
    - content_type # 必填,int,0系统端角色，1业务的角色
    - key # 必填,str,接口标识符
    - name # 必填,str,名称
    - permission_class # 必填,int,类别，tag标签，api接口权限
    - method # 选填,str,接口方法， 当permission_class=api时必须提供
    - parent_id # 选填,str,父级id
    - describes # 选填,str,角色说明
    - state # 选填,int,角色说明
    - hook_func # 选填,str,钩子方法
    """
    headers = {}
    files = {}
    params = {
        "content_type": 1,
        "key": "dfddd",
        "name": "(必填)名称",
        "permission_class": "api",
        "method": "get",
        # "parent_id": "(选填)父级id",
        "describes": "(选填)角色说明",
        "state": 1,
        "hook_func": "(选填)钩子方法",
    }
    return send(methed="post", url="/permission", params=params, headers=headers, files=files)


def put_permission(*args, **kwargs):
    """
    修改:修改权限名称，描述，状态，钩子

    参数:
    - content_type # 必填,int,0系统端角色，1业务的角色
    - key # 选填,str,接口标识符
    - name # 选填,str,名称
    - permission_class # 选填,int,类别，tag标签，api接口权限
    - method # 选填,str,接口方法， 当permission_class=api时必须提供
    - parent_id # 选填,str,父级id
    - describes # 选填,str,角色说明
    - state # 选填,int,角色说明
    - hook_func # 选填,str,钩子方法
    """
    headers = {}
    files = {}
    params = {
        "permission_id": 276,
        "content_type": 1,
        "name": "(选填)名称22",
        # "permission_class": "tag",
        "method": "post",
        "parent_id": 255,
        "describes": "(选填)角色说明",
        "state": 1,
        "hook_func": "(选填)钩子方法",
    }
    return send(methed="put", url="/permission", params=params, headers=headers, files=files)


def delete_permission(*args, **kwargs):
    """
    删除:

    参数:
    - permission_id # 必填,int,id
    """
    headers = {}
    files = {}
    params = {
        "permission_id": 275,
        "content_type": 1
    }
    return send(methed="delete", url="/permission", params=params, headers=headers, files=files)


def get_permission_list(*args, **kwargs):
    """
    权限条目清单:

    参数:
    - state # 选填,int,状态
    - content_type # 必填,int,0系统端权限条目，1业务端权限条目
    """
    headers = {}
    files = {}
    params = {
        "state": 1,
        "content_type": 0,
    }
    return send(methed="get", url="/permission/list", params=params, headers=headers, files=files)


def get_permission_enable(*args, **kwargs):
    """
    是否启用权限系统:
    """
    headers = {}
    files = {}
    params = {
        "content_type": 1
    }
    return send(methed="get", url="/permission/enable", params=params, headers=headers, files=files)


def post_permission_enable(*args, **kwargs):
    """
    启用停用:停用权限系统后，所有访问都不做拦截

    参数:
    - state # 必填,int,状态， 1启用，0停用
    """
    headers = {}
    files = {}
    params = {
        "state": 0,
        "content_type": 0,
    }
    return send(methed="post", url="/permission/enable", params=params, headers=headers, files=files)


def post_permission_loads(*args, **kwargs):
    """
    载入

    参数:
    """
    headers = {}
    files = {}
    params = {

    }
    return send(methed="post", url="/permission/loads", params=params, headers=headers, files=files)


if __name__ == "__main__":
    # 角色详情
    # get_permission_groups()
    # 新增角色
    # post_permission_groups()
    # 修改角色
    # put_permission_groups()
    # # 删除角色
    # delete_permission_groups()
    # 角色列表
    # get_permission_groups_list()
    # 属于该角色的用户
    # get_permission_groups_members()
    # 给用户指派角色
    # post_permission_groups_members()
    # # 编辑角色权限(单个)
    # post_permission_groups_check()
    # # 编辑角色权限（批量）
    post_permission_groups_checks()
    # # 商户的所有角色
    # get_permission_groups_company()
    # 分配角色
    # post_permission_groups_company()
    # # 移除角色
    # delete_permission_groups_company()
    # # 权限详情
    # get_permission()
    # # 新增
    # post_permission()
    # 修改
    # put_permission()
    # # 删除
    # delete_permission()
    # # 权限条目清单
    # get_permission_list()
    # # 是否启用权限系统
    # get_permission_enable()
    # # 启用停用
    # post_permission_enable()
    # 自动载入
    # post_permission_loads()


# -*- coding:utf-8 -*-
from lcyframe.libs.errors import ApiError as BaseError
from lcyframe.libs.errors import *
from falcon.status_codes import HTTP_BAD_REQUEST, HTTP_NOT_FOUND, HTTP_FORBIDDEN


class ApiError(BaseError):
    code = 1
    code_name = 'Api Runtime Error'
    message = 'Runtime api error occurred.'
    status = HTTP_BAD_REQUEST

    def __init__(self, message=None, code=None, code_name=None, *args, **kwargs):
        super(ApiError, self).__init__(*args, **kwargs)
        # if zh_message:
        #     self.zh_message = zh_message
        if message:
            self.message = message
        else:
            if hasattr(self, "zh_message") and self.zh_message:
                self.message = self.zh_message
        if code is not None:
            self.code = code
        if code_name is not None:
            self.code_name = code_name

    def to_dict(self, *args, **kwargs):
        return {
            'err_code': self.code,
            'err_name': self.code_name,
            'message': self.message
        }


class DemoObjectError(ApiError):
    """
    自定义错误对象
    """
    code = 1000
    code_name = 'DemoObjectError'
    message = 'DemoObjectError'
    zh_message = '自定义错误说明'

class ErrorSystemAPI(ApiError):
    """
    ******
    """
    code = 2000
    code_name = 'Api server is prohibition'
    zh_message = 'API服务暂停访问！'
    help = "系统升级中..."




#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseSchema
from bson.objectid import ObjectId
import random
import string
import time

class IdGeneratorSchema(BaseSchema):
    """
    自增字段表
    """
    collection = "id_generator"
    def __init__(self):
        self._id = ObjectId()
        self.uid = 100000  # ******
        self.sid = 2000
        self.pid = 3000





#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os
from kombu import Exchange, Queue
from kombu import serialization
# https://www.cnblogs.com/yunlongaimeng/p/11121294.html

# lcyframe框架下推荐使用celery=4.4.7。若不同版本的celery服务连接到同一个redis后端时，可能会引起冲突，work启动失败
# redis://:yourpassword@localhost:6379/0
redis_config = os.environ.app_config["redis_config"]
host = redis_config['host']
port = redis_config['port']
db = redis_config['db']
BROKER_URL = f"redis://{host}:{port}/{db}"
CELERY_RESULT_BACKEND = f"redis://{host}:{port}/{db}"

# serialization.registry._decoders.pop("application/x-python-serialize")
CELERY_TASK_SERIALIZER = 'json'     # pickle
CELERY_RESULT_SERIALIZER = 'json'
CELERY_ACCEPT_CONTENT = ['pickle', 'json']
# if os.name != "nt":
#     # Mac and Centos
#     CELERY_ACCEPT_CONTENT = ['application/json', ]
#     CELERY_TASK_SERIALIZER = 'json'
#     CELERY_RESULT_SERIALIZER = 'json'
# else:
#     # windows
#     CELERY_ACCEPT_CONTENT = ['pickle', ]
#     CELERY_TASK_SERIALIZER = 'pickle'
#     CELERY_RESULT_SERIALIZER = 'pickle'

# 任务失败或超时自动确认，默认为True
CELERY_ACKS_ON_FAILURE_OR_TIMEOUT = False

# 任务完成之后再确认
CELERY_ACKS_LATE = True

# worker进程崩掉之后拒绝确认
CELERY_REJECT_ON_WORKER_LOST = True

# 设置默认不存结果(保存在redis内),如果需要保持结果，请在任务中单独指定。@app.task(ignore_result=False)
CELERY_IGNORE_RESULT = True

# 设置结果的保存时间
CELERY_TASK_RESULT_EXPIRES = 24 * 3600

# worker执行任务以后不会自动释放内存，需要设置每个进程执行固定次数后销毁
CELERYD_MAX_TASKS_PER_CHILD = 10  # 每个worker执行了多少任务就会死掉，防止OOM

# 队列路由规则：routing_key、queue的名字规则
CELERY_QUEUES = (
    Queue("default", Exchange("default", type='direct'), routing_key="default"),
    Queue("celery", Exchange("celery", type='direct'), routing_key="celery"),
    Queue("for_task_A", Exchange("for_task_A", type='direct'), routing_key="for_task_A"),
    Queue("for_task_B", Exchange("for_task_B", type='direct'), routing_key="for_task_B")
)

# 批量设置文件内的所有事件的队列名称
CELERY_ROUTES = {
    'tasks.task_x': {"queue": "celery"},
    'tasks.taskA': {"queue": "for_task_A", "routing_key": "for_task_A"},
    'tasks.taskB': {"queue": "for_task_B", "routing_key": "for_task_B"}
}

# 新增加的定时任务部分：celery默认时区为世界标准时间，比东八区慢8小时
# https://www.cnblogs.com/belingud/p/11716303.html
CELERY_TIMEZONE = 'Asia/Shanghai'

# CELERYBEAT_SCHEDULE = {
#     'taskA_schedule' : {
#         'task':'tasks.taskA',
#         'schedule':2,
#         'args':(5,6)
#     },
#     'taskB_scheduler' : {
#         'task':"tasks.taskB",
#         "schedule":10,
#         "args":(10,20,30)
#     },
#     'add_schedule': {
#         "task":"tasks.add",
#         "schedule":5,
#         "args":(1,2)
#     }
# }



# coding=utf-8
import time, os, logging
import tornado
import tornado.ioloop
import tornadoredis
from tornado import gen
from .libs.mq_route import MqRoute
from .libs.hot_queue import key_for_name
from traceback import format_exc
import json

# 废弃
# 仅支持在tornado4.x版本工作，5.0以上删除了io_loop参数，导致报错：
# ERROR:root:TypeError("__init__() got an unexpected keyword argument 'io_loop'") waiting for 4s
# 链接池有缺陷，每次完成获取，需要手动释放：self.c.disconnect
# CONNECTION_POOL = tornadoredis.ConnectionPool(max_connections=500, wait_for_available=True)

class MqWorker(object):
    def __init__(self, **kwargs):

        self.ROOT = kwargs["ROOT"]
        self.mq_config = kwargs["mq_config"]
        self.name = self.mq_config["name"]
        self.workers_path = self.mq_config.get("workers_dir", ["works/mq"])
        self.c = tornadoredis.Client(host=self.mq_config.get("host", "127.0.0.1"),
                                     port=int(self.mq_config.get("port", 6379)),
                                     selected_db=int(self.mq_config.get("db", 0)))
        self.ioloop = tornado.ioloop.IOLoop.instance()
        self.name = key_for_name(self.name)
        self.wait = 1
        self.workers = MqRoute.get_workers(self.ROOT, self.workers_path)

    def connect(self):
        try:
            logging.debug("connect redis success!")
            self.c.connect()
            self.loop()
        except Exception as e:
            logging.error(repr(e) + " waiting for %ds" % self.wait)
            self.ioloop.add_timeout(time.time() + self.wait, self.connect)
            self.wait = min(self.wait * 2, 128)

    @gen.engine
    def loop(self):
        try:
            self.wait = 1
            res = yield gen.Task(self.c.blpop, self.name, 1)
            if res:
                msg = json.loads(str(res[self.name]))
                self.workers[msg["event"]](**msg)
                logging.debug("success finish job :%s" % repr(msg))
            self.ioloop.add_callback(self.loop)
        except Exception as e:
            logging.warning(format_exc())
            logging.error("%s loop error!" % repr(e))
            self.connect()

    def start(self):
        self.connect()
        self.ioloop.start()


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    kwargs = {"mq_config": {
        "workers_dir": ["workers"],
        "host": "127.0.0.1",
        "port": 6379,
        "name": "mq"
    }}
    wk = MqWorker(**kwargs)
    wk.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from bson.objectid import ObjectId
from base import BaseSchema
from lcyframe.libs import utils

class UserSchema(BaseSchema):
    """
    user
    用户
    """

    collection = "user"

    def __init__(self):
        self._id = ObjectId()
        self.uid = utils.random.randint(1, 50000000)
        self.sex = 1
        self.city = utils.random_int(5)
        self.age = utils.random.randint(20, 80)
        self.addr = utils.random_string(20)
        self.iphone = str(utils.random_int(10))
        self.name = utils.random_string(10)
        self.money = int(utils.random_int(5))





#!/usr/bin/env python
# -*- coding:utf-8 -*-

from bson.objectid import ObjectId
from base import BaseSchema
import time, datetime

class PermissionGroupSchema(BaseSchema):
    """
    permission
    """

    collection = "permission_group"

    def __init__(self):
        self._id = ObjectId()
        self.available = True   # ******
        self.permission_class = {}  # ******
        self.gid = 1    # ******ID
        self.name = "******2"
        self.pid = 1    # ******ID
        self.desc = ""  # ******
        self.create_at = int(time.time())
        self.update_at = int(time.time())

class PermissionClassSchema(BaseSchema):
    """
    permission
    """

    collection = "permission_class"

    def __init__(self):
        self._id = ObjectId()
        self.available = True   # ******
        self.bit_mp = {}  # ******bit
        self.key = 1    # ******ID
        self.name = "******"
        self.desc = ""  # ******
        self.create_at = int(time.time())
        self.update_at = int(time.time())



########################  RBAC  ######################

class GroupsSchema(BaseSchema):
    """
    角色
    """

    collection = "groups"

    def __init__(self):
        self.id = 0
        self.name = "角色名"
        self.state = 1
        self.describes = "描述"
        self.permissions = '{}'       # 已勾选的权限
        self.content_type = 1  # 1业务的权限，0系统端
        self.create_time = datetime.datetime.now()
        self.update_time = datetime.datetime.now()

class CompanyGroupsSchema(BaseSchema):
    """
    商户的角色
    """

    collection = "company_groups"

    def __init__(self):
        self.id = 0
        self.group_id = ""   # 已分配的角色id
        self.company_id = ""  # 商户id
        self.create_time = datetime.datetime.now()
        self.update_time = datetime.datetime.now()

class PermissionsSchema(BaseSchema):
    """
    权限条目
    """

    collection = "permissions"

    def __init__(self):
        self.id = 0
        self.name = "权限名"
        self.key = "权限键"
        self.method = ""  # 请求方法,get、post、put、delete
        self.parent_id = None  # 父类的id
        self.permission_class = ""  # 权限类别，tag标签，api接口权限，logic逻辑权限
        self.hook_func = ""  # 钩子函数
        self.state = 1  # 1启用0停用
        self.describes = ""
        self.content_type = 1  # 1业务的权限，0系统端
        self.create_time = datetime.datetime.now()
        self.update_time = datetime.datetime.now()
